////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   : EVE Online KillMail Sender
//  * Unit Name : kmsDebugLog
//  * Purpose   : ,      .
//  *           :      .
//  * Author    :  (Rouse_) 
//  * Copyright :  Fangorn Wizards Lab 1998 - 2009 ()
//  * Version   : 1.09
//  * Home Page : http://rouse.drkb.ru/kms/
//  ****************************************************************************
//

unit kmsDebugLog;

interface

uses
  Classes,
  SysUtils;

type
  TDebugLog = class
  strict private
    FLogPath, FLogName: String;
    FLogData: TStringList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure StartLog;
    procedure EndLog;
    procedure Add(const Value: String);
  end;

var
  DebugLog: TDebugLog;

implementation

{ TDebugLog }

procedure TDebugLog.Add(const Value: String);
begin
  FLogData.Add(DateTimeToStr(Now) + '> ' + Value);
end;

constructor TDebugLog.Create;
begin
  FLogPath := ExtractFilePath(ParamStr(0)) + 'DebugLog\';
  ForceDirectories(FLogPath);
  FLogData := TStringList.Create;
end;

destructor TDebugLog.Destroy;
begin
  FLogData.Free;
  inherited;
end;

procedure TDebugLog.EndLog;
begin
  if FLogName <> '' then
  begin
    Add('Log stopped');
    FLogData.SaveToFile(FLogPath + FLogName);
  end;
  FLogData.Clear;
end;

procedure TDebugLog.StartLog;
begin
  FLogName := StringReplace(DateTimeToStr(Now), ':', '_', [rfReplaceAll]) + '.txt';
  EndLog;
  Add('Log started');
end;

{$IFDEF DEBUG_LOG}
initialization

  DebugLog := TDebugLog.Create;

finalization

  DebugLog.Free;
{$ENDIF}

end.
