////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   : EVE Online KillMail Sender
//  * Unit Name : kmsHistory
//  * Purpose   :   THistoryProvider,
//  *           :       
//  * Author    :  (Rouse_) 
//  * Copyright :  Fangorn Wizards Lab 1998 - 2009 ()
//  * Version   : 1.09
//  * Home Page : http://rouse.drkb.ru/kms/
//  ****************************************************************************
//

unit kmsHistory;

interface

uses
  SysUtils,
  Classes;

const
  HeaderLength = 2;

type
  THistoryData = record
    MailPath: String;
    Victim: String;
    Loose: Boolean;
    SuccessKBList: String;
  end;

  THistoryProvider = class
  strict private
    FHistroyPath: String;
    FHistory: array of THistoryData;
    FHideSendedMails: Boolean;
    FFilter: TList;
    FKBList, FNotSendLooseKB: TStringList;
    FStartMailName: Integer;
    FAllVictims: TStringList;
    procedure SetHideSendedMails(const Value: Boolean);
    function GetMail(Index: Integer): THistoryData;
    function GetCount: Integer;
  protected
    procedure InitMails;
  public
    constructor Create(StartMailName: Integer);
    destructor Destroy; override;
    procedure ReinitMails;
    function GetVictimIndex(const Value: String): Integer;
    procedure SetCurrentKBList(const AllKB, NotSendLooseKB: TStringList);
    procedure AddMail(const Mail, Victim: String;
      const Loose: Boolean; const SuccessSended: TStringList);
    procedure ChangeMail(const MailIndex: Integer;
      const SuccessSended: TStringList);
    property HideSuccessSendedMails: Boolean read FHideSendedMails
      write SetHideSendedMails;
    property Count: Integer read GetCount;
    property Mail[Index: Integer]: THistoryData read GetMail; default;
    property LastMailName: Integer read FStartMailName;
  end;

implementation

const
  LooseState: array [Boolean] of String = ('kill', 'loose');

{ THistoryProvider }

procedure THistoryProvider.AddMail(const Mail, Victim: String;
  const Loose: Boolean;
  const SuccessSended: TStringList);
var
  S: TStringList;
  Index: Integer;
begin
  Index := GetVictimIndex(Victim);
  if Index >= 0 then
  begin
    ChangeMail(Index, SuccessSended);
    Exit;
  end;
  SetLength(FHistory, Count + 1);
  Inc(FStartMailName);
  FHistory[Count].MailPath := FHistroyPath +
    IntToStr(FStartMailName) + '.mail';
  FHistory[Count].SuccessKBList := SuccessSended.Text;
  FHistory[Count].Loose := Loose;
  FHistory[Count].Victim := Victim;
  FFilter.Add(Pointer(Count));
  S := TStringList.Create;
  try
    S.Text := Mail;
    S.SaveToFile(FHistory[Count - 1].MailPath);
    S.Clear;
    S.Add(IntToStr(HeaderLength));
    S.Add(Victim);
    S.Add(LooseState[Loose]);
    S.Add(SuccessSended.Text);
    S.SaveToFile(FHistroyPath +
      IntToStr(FStartMailName) + '.dat');
    FAllVictims.Add(Victim);
  finally
    S.Free;
  end;
end;

procedure THistoryProvider.ChangeMail(const MailIndex: Integer;
  const SuccessSended: TStringList);
var
  S, SS: TStringList;
  I: Integer;
begin
  S := TStringList.Create;
  try
    S.Add(IntToStr(HeaderLength));
    S.Add(FHistory[MailIndex].Victim);
    S.Add(LooseState[FHistory[MailIndex].Loose]);
    SS := TStringList.Create;
    try
      SS.Sorted := True;
      SS.Duplicates := dupIgnore;
      SS.Text := SuccessSended.Text + FHistory[MailIndex].SuccessKBList;
      for I := SS.Count - 1 downto 0 do
        if SS[I] = '' then
          SS.Delete(I);
      S.Add(SS.Text);
      FHistory[MailIndex].SuccessKBList := SS.Text;      
    finally
      SS.Free;
    end;
    S.SaveToFile(ChangeFileExt(FHistory[MailIndex].MailPath, '.dat'));
  finally
    S.Free;
  end;
end;

constructor THistoryProvider.Create(StartMailName: Integer);
begin
  FFilter := TList.Create;
  FKBList := TStringList.Create;
  FNotSendLooseKB := TStringList.Create;
  FStartMailName := StartMailName;
  FHistroyPath := ExtractFilePath(ParamStr(0)) + 'PostData\';
  FAllVictims := TStringList.Create;
  InitMails;
end;

destructor THistoryProvider.Destroy;
begin
  FAllVictims.Free;
  FNotSendLooseKB.Free;
  FKBList.Free;
  FFilter.Free;
  inherited;
end;

function THistoryProvider.GetCount: Integer;
begin
  Result := FFilter.Count;
end;

function THistoryProvider.GetMail(Index: Integer): THistoryData;
begin
  Result := FHistory[Integer(FFilter[Index])];
end;

function THistoryProvider.GetVictimIndex(const Value: String): Integer;
begin
  Result := FAllVictims.IndexOf(Value);
end;

procedure THistoryProvider.InitMails;
var
  DataHeaderCount, CurrMailName: Integer;
  SR: TSearchRec;
  MailPath: String;
  MailData: TStringList;
begin
  FFilter.Clear;
  SetLength(FHistory, 0);
  FHistroyPath := ExtractFilePath(ParamStr(0)) + 'PostData\';
  MailData := TStringList.Create;
  try
    if FindFirst(FHistroyPath + '*.dat', faAnyFile, SR) = 0 then
    try
      repeat
        MailData.LoadFromFile(FHistroyPath + SR.Name);
        MailPath := FHistroyPath + ChangeFileExt(SR.Name, '.mail');
        if not FileExists(MailPath) then
          Continue;

        if TryStrToInt(Copy(SR.Name, 1,Length(SR.Name) - 4), CurrMailName) then
        begin
          if FStartMailName < CurrMailName then
            FStartMailName := CurrMailName;
        end;
                                               
        if not TryStrToInt(MailData[0], DataHeaderCount) then
          Continue;
        if (DataHeaderCount < HeaderLength) or (MailData.Count < DataHeaderCount) then
          Continue;
        MailData.Delete(0);
        SetLength(FHistory, Count + 1);
        FHistory[Count].MailPath := MailPath;
        FHistory[Count].Victim := MailData[0];
        MailData.Delete(0);
        FHistory[Count].Loose := LowerCase(MailData[0]) = 'loose';
        MailData.Delete(0);
        Dec(DataHeaderCount, HeaderLength);
        while DataHeaderCount > 0 do
        begin
          MailData.Delete(0);
          Dec(DataHeaderCount);
        end;
        FHistory[Count].SuccessKBList := MailData.Text;
        FAllVictims.Add(FHistory[Count].Victim);
        FFilter.Add(Pointer(Count));
      until FindNext(SR) <> 0;
    finally
      FindClose(SR);
    end;
  finally
    MailData.Free;
  end;
end;

procedure THistoryProvider.ReinitMails;
begin
  InitMails;
end;

procedure THistoryProvider.SetCurrentKBList(
  const AllKB, NotSendLooseKB: TStringList);
begin
  FKBList.Text := AllKB.Text;
  FNotSendLooseKB.Text := NotSendLooseKB.Text;
  if HideSuccessSendedMails then
  begin
    FHideSendedMails := False;
    HideSuccessSendedMails := True;
  end;
end;

procedure THistoryProvider.SetHideSendedMails(const Value: Boolean);
var
  I, A, Index: Integer;
  SuccesedSended, ExcludedList: TStringlist;
begin
  if FHideSendedMails <> Value then
  begin
    FHideSendedMails := Value;
    if Value then
    begin
      FFilter.Clear;
      SuccesedSended := TStringList.Create;
      try
        ExcludedList := TStringList.Create;
        try
          for I := 0 to Length(FHistory) - 1 do
          begin
            ExcludedList.Text := FKBList.Text;

            // if loose, delete untrusted kb
            if FHistory[I].Loose then
            begin
              for A := 0 to FNotSendLooseKB.Count - 1 do
              begin
                Index := ExcludedList.IndexOf(FNotSendLooseKB[A]);
                if Index >= 0 then
                  ExcludedList.Delete(Index);
              end;
            end;

            SuccesedSended.Text := FHistory[I].SuccessKBList;
            for A := 0 to SuccesedSended.Count - 1 do
            begin
              Index := ExcludedList.IndexOf(SuccesedSended[A]);
              if Index >= 0 then
                ExcludedList.Delete(Index);
            end;

            if ExcludedList.Count > 0 then
              FFilter.Add(Pointer(I));
          end;
        finally
          ExcludedList.Free;
        end;
      finally
        SuccesedSended.Free;
      end;                  
    end
    else
    begin
      FFilter.Count := Length(FHistory);
      for I := 0 to FFilter.Count - 1 do
        FFilter[I] := Pointer(I);
    end;
  end;
end;

end.
