////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   : EVE Online KillMail Sender
//  * Unit Name : kmsKBConfig
//  * Purpose   :      ,
//  *           :    
//  *           :  .
//  * Author    :  (Rouse_) 
//  * Copyright :  Fangorn Wizards Lab 1998 - 2009 ()
//  * Version   : 1.09
//  * Home Page : http://rouse.drkb.ru/kms/
//  ****************************************************************************
//

unit kmsKBConfig;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls, ToolWin, kmsSettings, ExtDlgs,
  ImgList;

type
  TkmsKbConfigDialog = class(TForm)
    gbKBList: TGroupBox;
    lvKB: TListView;
    gbCorpDetail: TGroupBox;
    edAlliyDescr: TLabeledEdit;
    edCorpDescr: TLabeledEdit;
    pnKB: TPanel;
    edKBDescr: TLabeledEdit;
    Bevel1: TBevel;
    edImagePath: TEdit;
    btnBrowse: TSpeedButton;
    lblImgPath: TLabel;
    lblTpl: TLabel;
    cbTemplate: TComboBox;
    lblTplEnv: TLabel;
    btnCancel: TButton;
    btnOk: TButton;
    memEnvironment: TMemo;
    btnApply: TButton;
    cbSendLooses: TCheckBox;
    OpenPictureDialog: TOpenPictureDialog;
    KBImageList: TImageList;
    edFactionDescr: TLabeledEdit;
    gbProxy: TGroupBox;
    edProxy: TLabeledEdit;
    edPort: TLabeledEdit;
    edUser: TLabeledEdit;
    edPwd: TLabeledEdit;
    lblHint: TLabel;
    Bevel2: TBevel;
    Panel: TPanel;
    btnAdd: TButton;
    btnDel: TButton;
    cbKBDisabled: TCheckBox;
    Bevel3: TBevel;
    procedure FormActivate(Sender: TObject);
    procedure lvKBChange(Sender: TObject; Item: TListItem; Change: TItemChange);
    procedure edKBDescrChange(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure btnBrowseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnDelClick(Sender: TObject);
  private
    LockUpdate: Boolean;
    FSelectedIndex: Integer;
    EmptyIcon: TBitmap;
    KBIcon: TIcon;
    procedure SetSelectedIndex(const Value: Integer);
    property SelectedIndex: Integer read FSelectedIndex write SetSelectedIndex;
    procedure UpdateEnabledState(EnableState: Boolean);
    procedure AppendDataToCurrentItem;
    procedure InitFromSettings;
  public
    Settings: TSettingProvider;
  end;

var
  kmsKbConfigDialog: TkmsKbConfigDialog;

implementation

uses
  kmsLangSettings;

{$R *.dfm}

procedure TkmsKbConfigDialog.AppendDataToCurrentItem;
begin
  Settings.AllianceDescription := edAlliyDescr.Text;
  Settings.CorporationDescription := edCorpDescr.Text;
  Settings.FactionDescription := edFactionDescr.Text;
  Settings.ProxyServer := edProxy.Text;
  Settings.ProxyPort := edPort.Text;
  Settings.ProxyLogin := edUser.Text;
  Settings.ProxyPassword := edPwd.Text;
  if edKBDescr.Enabled then
  begin
    lvKB.Selected.Caption := edKBDescr.Text;
    Settings.KBList[SelectedIndex].KBTitle := edKBDescr.Text;
    Settings.KBList[SelectedIndex].KBImage := edImagePath.Text;
    Settings.KBList[SelectedIndex].Template :=
      cbTemplate.Items[cbTemplate.ItemIndex];
    Settings.KBList[SelectedIndex].EnvironmentList := memEnvironment.Text;
    Settings.KBList[SelectedIndex].SendLooses := cbSendLooses.Checked;
    Settings.KBList[SelectedIndex].Enabled := not cbKBDisabled.Checked;
  end;
end;

procedure TkmsKbConfigDialog.btnAddClick(Sender: TObject);
var
  NewKBCaption: String;
begin
  NewKBCaption := Format(Settings.GetLocalisedString(lsNewKB),
    [lvKB.Items.Count + 1]);
  LockUpdate := True;
  try
    Settings.KBList.Add.KBTitle := NewKBCaption;
    KBImageList.Add(EmptyIcon, nil);
    with lvKB.Items.Add do
    begin
      Caption := NewKBCaption;
      Selected := True;
      StateIndex := lvKB.Items.Count - 1;
      SelectedIndex := lvKB.Items.Count - 1;
      btnApply.Enabled := True;
      btnOk.Enabled := True;
    end;
  finally
    LockUpdate := False;
  end;
end;

procedure TkmsKbConfigDialog.btnApplyClick(Sender: TObject);
begin
  Settings.SaveSetting;
  btnApply.Enabled := False;
end;

procedure TkmsKbConfigDialog.btnBrowseClick(Sender: TObject);
begin
  if OpenPictureDialog.Execute then
  begin
    try
      KBIcon.LoadFromFile(OpenPictureDialog.FileName);
      edImagePath.Text := OpenPictureDialog.FileName;
    except
      on E: Exception do
      begin
        Application.MessageBox(
          PChar(Settings.GetLocalisedString(lsIconError)),
          PChar(Settings.GetLocalisedString(lsError)), MB_ICONERROR);
        if not FileExists(edImagePath.Text) then
          edImagePath.Text := '';
        Exit;
      end;
    end;
    KBImageList.ReplaceIcon(SelectedIndex, KBIcon);
  end;
end;

procedure TkmsKbConfigDialog.btnCancelClick(Sender: TObject);
begin
  Settings.LoadSetting;
  ModalResult := mrCancel;
end;

procedure TkmsKbConfigDialog.btnDelClick(Sender: TObject);
begin
  if Application.MessageBox(
    PChar(Format(Settings.GetLocalisedString(lsDelConfirm),
    [lvKB.Selected.Caption])),
    PChar(Settings.GetLocalisedString(lsDel)),
    MB_ICONQUESTION or MB_YESNO or MB_DEFBUTTON2) = IDYES then
  begin
    Settings.KBList.Delete(SelectedIndex);
    InitFromSettings;
    btnApply.Enabled := True;
    btnOk.Enabled := True;    
  end;
end;

procedure TkmsKbConfigDialog.btnOkClick(Sender: TObject);
begin
  btnApplyClick(nil);
  ModalResult := mrOk;
end;

procedure TkmsKbConfigDialog.edKBDescrChange(Sender: TObject);
begin
  if not LockUpdate then
  begin
    btnApply.Enabled := True;
    btnOk.Enabled := True;
    AppendDataToCurrentItem;
    if cbTemplate.ItemIndex >= 0 then
      lblHint.Caption :=
        Settings.KBTemplates[cbTemplate.ItemIndex].GetData(
          'empty_environments_hint', IntToStr(Settings.LanguageID));
  end;
end;

procedure TkmsKbConfigDialog.FormActivate(Sender: TObject);
begin
  InitFromSettings;
end;

procedure TkmsKbConfigDialog.FormCreate(Sender: TObject);
begin
  EmptyIcon := TBitmap.Create;
  EmptyIcon.Width := 16;
  EmptyIcon.Height := 16;
  KBIcon := TIcon.Create;
end;

procedure TkmsKbConfigDialog.FormDestroy(Sender: TObject);
begin
  KBIcon.Free;
  EmptyIcon.Free;
end;

procedure TkmsKbConfigDialog.InitFromSettings;
var
  I: Integer;
begin
  LockUpdate := True;
  try
    lvKB.Clear;
    KBImageList.Clear;
    for I := 0 to Settings.KBList.Count - 1 do
    begin
      with lvKB.Items.Add do
      begin
        if FileExists(Settings.KBList[I].KBImage) then
        begin
          try
            KBIcon.LoadFromFile(Settings.KBList[I].KBImage);
            KBImageList.AddIcon(KBIcon);
          except
            Settings.KBList[I].KBImage := '';
            KBImageList.Add(EmptyIcon, nil);
          end;
        end
        else
          KBImageList.Add(EmptyIcon, nil);
        Caption := Settings.KBList[I].KBTitle;
        StateIndex := I;
      end;
    end;
    cbTemplate.Clear;
    for I := 0 to Settings.KBTemplates.Count - 1 do
      cbTemplate.Items.Add(Settings.KBTemplates[I].Description);
    edAlliyDescr.Text := Settings.AllianceDescription;
    edCorpDescr.Text := Settings.CorporationDescription;
    edFactionDescr.Text := Settings.FactionDescription;
    edProxy.Text := Settings.ProxyServer;
    edPort.Text := Settings.ProxyPort;
    edUser.Text := Settings.ProxyLogin;
    edPwd.Text := Settings.ProxyPassword;
    edKBDescr.Text := '';
    edImagePath.Text := '';
    cbTemplate.ItemIndex := -1;
    memEnvironment.Text := '';
    cbSendLooses.Checked := True;
    btnApply.Enabled := False;
    btnOk.Enabled := False;
    SelectedIndex := -1;
    if Settings.KBList.Count > 0 then
    begin
      lvKB.Items[0].Selected := True;
      SelectedIndex := 0;
    end
    else
      UpdateEnabledState(False);
  finally
    LockUpdate := False;
  end;
end;

procedure TkmsKbConfigDialog.lvKBChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  if not LockUpdate then
  begin
    LockUpdate := True;
    try
      SelectedIndex := Item.Index;
    finally
      LockUpdate := False;
    end;
  end;
end;

procedure TkmsKbConfigDialog.SetSelectedIndex(const Value: Integer);
var
  EnableState: Boolean;
begin
  if SelectedIndex <> Value then
  begin
    FSelectedIndex := Value;
    EnableState := not ((SelectedIndex < 0) or
      (SelectedIndex > lvKB.Items.Count - 1));
    if EnableState then
    begin
      edKBDescr.Text := Settings.KBList[SelectedIndex].KBTitle;
      edImagePath.Text := Settings.KBList[SelectedIndex].KBImage;
      cbTemplate.ItemIndex := cbTemplate.Items.IndexOf(
        Settings.KBList[SelectedIndex].Template);
      if cbTemplate.ItemIndex >= 0 then      
        lblHint.Caption :=
          Settings.KBTemplates[cbTemplate.ItemIndex].GetData(
            'empty_environments_hint', IntToStr(Settings.LanguageID));
      // BUG-BUG!! sLineBreak duplicated
      memEnvironment.Text :=
        StringReplace(StringReplace(
          Settings.KBList[SelectedIndex].EnvironmentList,
          #10, sLineBreak, [rfReplaceAll]),
          sLineBreak + sLineBreak, sLineBreak, [rfReplaceAll]);
      cbSendLooses.Checked := Settings.KBList[SelectedIndex].SendLooses;
      cbKBDisabled.Checked := not Settings.KBList[SelectedIndex].Enabled;
    end;
    UpdateEnabledState(EnableState);
  end;
end;

procedure TkmsKbConfigDialog.UpdateEnabledState(EnableState: Boolean);
begin
  edKBDescr.Enabled := EnableState;
  edImagePath.Enabled := EnableState;
  cbTemplate.Enabled := EnableState;
  memEnvironment.Enabled := EnableState;
  cbSendLooses.Enabled := EnableState;
  btnBrowse.Enabled := EnableState;
  btnDel.Enabled := EnableState;
end;

end.
