////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   : EVE Online KillMail Sender
//  * Unit Name : kmsLangSettings
//  * Purpose   : ,    
//  *           :   ,
//  *           :  -     
//  *           :     
//  * Author    :  (Rouse_) 
//  * Copyright :  Fangorn Wizards Lab 1998 - 2009 ()
//  * Version   : 1.09
//  * Home Page : http://rouse.drkb.ru/kms/
//  ****************************************************************************
//

unit kmsLangSettings;

interface

uses
  Windows,
  Messages,
  Classes,
  SysUtils,
  IniFiles,
  Menus;

type
  TLocalisedString = (lsNewKB, lsError, lsIconError, lsDel, lsDelConfirm,
    lsPSError, lsPSDone, lsConfirm, lsLoosoCongirm, lsParserError,
    lsKMNotFound, lsHeaderError, lsTryToSend, lsSendError, lsSendDone,
    lsWaiting, lsInvalidPwd1, lsInvalidPwd2, lsDuplicate1, lsDuplicate2,
    lsError1, lsError2, lsDone1, lsDone2, lsUnknown1, lsUnknown2,
    lsMoveToBackup, lsUnknownAnswer, lsTrustedKB, lsUntrustedKB);
  TLocalisedStrings = array [TLocalisedString] of String;

  TLanguageProvider = class
  private
    FLangMenus: TStringList;
    FLangFolderPath: String;
    FLayout: Integer;
    FLocalisedStrings: TLocalisedStrings;
    FHelpFile: String;
    procedure SetLayout(const Value: Integer);
  protected
    function GetItemIndexByLayout(const Value: Integer): Integer;
    procedure InitializeLanguages;
    procedure LangMenuClick(Sender: TObject);
  public
    constructor Create;
    destructor Destroy; override;
    property Layout: Integer read FLayout write SetLayout;
    property LocalisedStrings: TLocalisedStrings read FLocalisedStrings;
    property HelpFile: String read FHelpFile;
  end;

implementation

uses
  kmsMainDlg,
  kmsKBConfig;

{ TLanguageProvider }

constructor TLanguageProvider.Create;
begin
  FLangFolderPath := ExtractFilePath(ParamStr(0)) + 'Language\';
  FLangMenus := TStringList.Create;
  InitializeLanguages;
end;

destructor TLanguageProvider.Destroy;
var
  I: Integer;
begin
  for I := 0 to FLangMenus.Count - 1 do
    TMenuItem(FLangMenus.Objects[I]).Free;
  FLangMenus.Free;
  inherited;
end;

function TLanguageProvider.GetItemIndexByLayout(const Value: Integer): Integer;
var
  I: Integer;
begin
  for I := 0 to FLangMenus.Count - 1 do
    if TMenuItem(FLangMenus.Objects[I]).Tag = Value then
    begin
      Result := I;
      Exit;
    end;
  Result := -1;    
end;

procedure TLanguageProvider.InitializeLanguages;
var
  SR: TSearchRec;
  IniFile: TIniFile;
  LangMenu: TMenuItem;
begin                  
  if FindFirst(FLangFolderPath + '*.ini', faAnyFile, SR) = 0 then
  try
    kmsMainDialog.mnuLanguage.Clear;
    repeat
      IniFile := TIniFile.Create(FLangFolderPath + SR.Name);
      try
        LangMenu := TMenuItem.Create(kmsMainDialog.mnuLanguage);
        LangMenu.Caption := IniFile.ReadString('main', 'description', '');
        LangMenu.Tag := IniFile.ReadInteger('main', 'layout', 0);
        LangMenu.RadioItem := True;
        LangMenu.GroupIndex := 1;
        LangMenu.OnClick := LangMenuClick;
        FLangMenus.AddObject(SR.Name, LangMenu);
        kmsMainDialog.mnuLanguage.Add(LangMenu);
      finally
        IniFile.Free;
      end;
    until FindNext(SR) <> 0;
  finally
    FindClose(SR);
  end;
end;

procedure TLanguageProvider.LangMenuClick(Sender: TObject);
begin
  SetLayout(TMenuItem(Sender).Tag);
  kmsMainDialog.SaveLayout;
end;

procedure TLanguageProvider.SetLayout(const Value: Integer);
const
  Indent: TLocalisedStrings = ('newkb', 'error', 'icon_error',
    'del', 'del_confirm', 'parser_status_error', 'parser_status_done',
    'confirm', 'send_loose_confirm', 'parser_err', 'km_notfound',
    'hdr_error', 'trytosend', 'send_error', 'send_done',
    'report_waiting', 'report_invalitpwd', 'log_invalidpwd',
    'report_duplicate', 'log_duplicate', 'report_error', 'log_error',
    'report_done', 'log_done', 'report_unknown', 'log_unknown',
    'mov_tobackup', 'unknown_answer', 'trusted', 'untrusted');
var
  ItemIndex: Integer;
  IniFile: TIniFile;
  A: TLocalisedString;
begin
  if FLayout <> Value then
  begin
    FLayout := Value;
    ItemIndex := GetItemIndexByLayout(Value);
    if ItemIndex >= 0 then
    begin
      IniFile := TIniFile.Create(FLangFolderPath + FLangMenus[ItemIndex]);
      try
        FHelpFile := ExtractFilePath(ParamStr(0)) +
          IniFile.ReadString('main', 'helpfile', '');
        kmsMainDialog.mnuOpenHelp.Enabled := FileExists(HelpFile);
        kmsMainDialog.mnuFile.Caption :=
          IniFile.ReadString('send_dialog', 'mnu_file', '');
        kmsMainDialog.mnuSetting.Caption :=
          IniFile.ReadString('send_dialog', 'mnu_setting', '');
        kmsMainDialog.tbSetting.Hint := kmsMainDialog.mnuSetting.Caption;
        kmsMainDialog.mnuLanguage.Caption :=
          IniFile.ReadString('send_dialog', 'mnu_language', '');
        kmsMainDialog.mnuShowHistory.Caption :=
          IniFile.ReadString('send_dialog', 'mnu_showhistory', '');
        kmsMainDialog.tbShowHistory.Hint := kmsMainDialog.mnuShowHistory.Caption;
        kmsMainDialog.mnu_gsenable.Caption :=
          IniFile.ReadString('send_dialog', 'mnu_enablegs', '');
        kmsMainDialog.tbGSEnable.Hint := kmsMainDialog.mnu_gsenable.Caption;
        kmsMainDialog.mnuExit.Caption :=
          IniFile.ReadString('send_dialog', 'mnu_exit', '');
        kmsMainDialog.tbExit.Hint := kmsMainDialog.mnuExit.Caption;
        kmsMainDialog.mnuKillMail.Caption :=
          IniFile.ReadString('send_dialog', 'mnu_killmail', '');
        kmsMainDialog.mnuPastefromClipboard.Caption :=
          IniFile.ReadString('send_dialog', 'mnu_paste', '');
        kmsMainDialog.tbPaste.Hint :=
          kmsMainDialog.mnuPastefromClipboard.Caption;
        kmsMainDialog.mnuHelp.Caption :=
          IniFile.ReadString('send_dialog', 'mnu_help', '');
        kmsMainDialog.mnuOpenHelp.Caption :=
          IniFile.ReadString('send_dialog', 'mnu_open_help', '');
        kmsMainDialog.mnuAbout.Caption :=
          IniFile.ReadString('send_dialog', 'mnu_about', '');
        kmsMainDialog.mnuPaste.Caption :=
          IniFile.ReadString('send_dialog', 'mnu_paste', '');
        kmsMainDialog.mnuGroupSend1.Caption :=
          IniFile.ReadString('send_dialog', 'pmnu_groupsend', '');
        kmsMainDialog.mnuGroupSend.Caption :=
          IniFile.ReadString('send_dialog', 'pmnu_groupsend', '');
        kmsMainDialog.tbGroupSend.Hint := kmsMainDialog.mnuGroupSend.Caption;          
        kmsMainDialog.mnuCopy.Caption :=
          IniFile.ReadString('send_dialog', 'pmnu_copy', '');
        kmsMainDialog.pmHistoryBackup.Caption :=
          IniFile.ReadString('send_dialog', 'pmnu_backup', '');
        kmsMainDialog.mnuOpenKBInBrowser.Caption :=
          IniFile.ReadString('send_dialog', 'pmnu_openkb', '');
        kmsMainDialog.acReadyToSend.Caption :=
          IniFile.ReadString('send_dialog', 'action_send', '');
        kmsMainDialog.acReadyToSend.Hint := kmsMainDialog.acReadyToSend.Caption;
        kmsMainDialog.acStop.Caption :=
          IniFile.ReadString('send_dialog', 'action_stop', '');
        kmsMainDialog.acStop.Hint := kmsMainDialog.acStop.Caption;
        kmsMainDialog.gbGroupSend.Caption :=
          IniFile.ReadString('send_dialog', 'gb_groupsend', '');
        kmsMainDialog.lvGroupSend.Column[0].Caption :=
          IniFile.ReadString('send_dialog', 'col_path', '');
        kmsMainDialog.lvGroupSend.Column[1].Caption :=
          IniFile.ReadString('send_dialog', 'col_stat', '');
        kmsMainDialog.lvGroupSend.Column[2].Caption :=
          IniFile.ReadString('send_dialog', 'col_sendstat', '');
        kmsMainDialog.gbKillmail.Caption :=
          IniFile.ReadString('send_dialog', 'gb_kmdetail', '');
        kmsMainDialog.gbLog.Caption :=
          IniFile.ReadString('send_dialog', 'gb_sendlog', '');
        kmsMainDialog.gbKBList.Caption :=
          IniFile.ReadString('send_dialog', 'gb_kblist', '');
        kmsMainDialog.gbHistory.Caption :=
          IniFile.ReadString('send_dialog', 'gb_history', '');
        kmsMainDialog.lvKBList.Column[0].Caption :=
          IniFile.ReadString('send_dialog', 'col_title', '');
        kmsMainDialog.lvKBList.Column[1].Caption :=
          IniFile.ReadString('send_dialog', 'col_status', '');
        kmsMainDialog.rbShowAll.Caption :=
          IniFile.ReadString('send_dialog', 'rb_shoall', '');
        kmsMainDialog.rbShowSended.Caption :=
          IniFile.ReadString('send_dialog', 'rb_showsended', '');
        TMenuItem(FLangMenus.Objects[ItemIndex]).Checked := True;

        kmsKbConfigDialog.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'caption', '');
        kmsKbConfigDialog.gbCorpDetail.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'gb_corpdetail', '');
        kmsKbConfigDialog.edCorpDescr.EditLabel.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'ed_corpedsc', '');
        kmsKbConfigDialog.edAlliyDescr.EditLabel.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'ed_allydsc', '');
        kmsKbConfigDialog.edFactionDescr.EditLabel.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'ed_factiondsc', '');
        kmsKbConfigDialog.gbProxy.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'gb_proxy', '');
        kmsKbConfigDialog.edProxy.EditLabel.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'ed_proxy', '');
        kmsKbConfigDialog.edPort.EditLabel.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'ed_port', '');
        kmsKbConfigDialog.edUser.EditLabel.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'ed_user', '');
        kmsKbConfigDialog.edPwd.EditLabel.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'ed_pwd', '');
        kmsKbConfigDialog.gbKBList.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'gb_kblist', '');
        kmsKbConfigDialog.btnAdd.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'btn_newkb', '');
        kmsKbConfigDialog.btnDel.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'btn_delkb', '');
        kmsKbConfigDialog.pnKB.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'pn_kbparam', '');
        kmsKbConfigDialog.edKBDescr.EditLabel.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'ed_dsc', '');
        kmsKbConfigDialog.lblImgPath.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'ed_img', '');
        kmsKbConfigDialog.btnBrowse.Hint :=
          IniFile.ReadString('kbconfig_dialog', 'hint_browse', '');
        kmsKbConfigDialog.lblTpl.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'ed_tpl', '');
        kmsKbConfigDialog.lblTplEnv.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'ed_tplenv', '');
        kmsKbConfigDialog.cbSendLooses.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'cb_sendloos', '');
        kmsKbConfigDialog.btnApply.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'btn_apply', '');
        kmsKbConfigDialog.btnOk.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'btn_ok', '');
        kmsKbConfigDialog.btnCancel.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'btn_cancel', '');
        // 18.02.2009 Rouse_
        // ver 1.0.6 (New property for disabling killboard)
        kmsKbConfigDialog.cbKBDisabled.Caption :=
          IniFile.ReadString('kbconfig_dialog', 'cb_kbdisable', '');

        for A := Low(TLocalisedString) to High(TLocalisedString) do
          FLocalisedStrings[A] :=
            IniFile.ReadString('locales', Indent[A], '');

        PostMessage(kmsMainDialog.Handle, WM_USER, kmsMainDialog.Handle, 0);
      finally
        IniFile.Free;
      end;
    end;
  end;
end;

end.
