////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   : EVE Online KillMail Sender
//  * Unit Name : kmsMailParser
//  * Purpose   : ,    ,
//  *           :      (  )
//  * Author    :  (Rouse_) 
//  * Copyright :  Fangorn Wizards Lab 1998 - 2009 ()
//  * Version   : 1.09
//  * Home Page : http://rouse.drkb.ru/kms/
//  ****************************************************************************
//

unit kmsMailParser;

interface

uses
  Classes,
  SysUtils;

type
  TMailParserStatus = (psKill, psLoose, psNPC, psInvalid);
  TLoosDetect = (ldInFaction, ldInAlliance, ldInCorporate);
  TLoosDetects = set of TLoosDetect;

  TKillMailData = record
    KillDate: String;
    Victim: String;
    Status: TMailParserStatus;
    LoosDetect: TLoosDetects;
  end;

  TMailParser = class
  private
    FMail: TStringList;
    FFaction: String;
    FCorporation: String;
    FAlliance: String;
  public
    constructor Create;
    destructor Destroy; override;
    procedure SetCurrentData(const Faction, Alliance, Corporation: String);
    function ParseMail(const Value: String): TKillMailData;
  end;

var
  MailParser: TMailParser;

implementation

{$IFDEF DEBUG_LOG}
uses
  kmsDebugLog;
{$ENDIF}

{ TMailParser }

constructor TMailParser.Create;
begin
  FMail := TStringList.Create;
end;

destructor TMailParser.Destroy;
begin
  FMail.Free;
  inherited;
end;

{$IFDEF DEBUG_LOG}

function TMailParser.ParseMail(const Value: String): TKillMailData;

  procedure TestLine(const Line, Indent, IndentValue: String;
    LooseDetect: TLoosDetect);
  begin
    if Copy(Line, 1, Length(Indent)) = Indent then
      if Trim(Copy(Line, Length(Indent) + 1, Length(Line))) = IndentValue then
        if Result.Status <> psInvalid then
        begin
          DebugLog.Add('Found indent: "' + Indent + '", value = "' +
            IndentValue + '", Result.Status = psLoose');
          Result.Status := psLoose;
          Include(Result.LoosDetect, LooseDetect);
        end;
  end;

const
  INV = 'involved parties';
  VIC = 'victim:';
  CORP = 'corp:';
  ALLI = 'alliance:';
  FACTION = 'faction:';

  LFS = 'LocaleFormatSettings: ' + sLineBreak +
    'CurrencyFormat: %d' + sLineBreak +
    'NegCurrFormat: %d' + sLineBreak +
    'ThousandSeparator: %s' + sLineBreak +
    'DecimalSeparator: %s' + sLineBreak +
    'CurrencyDecimals: %d' + sLineBreak +
    'DateSeparator: %s' + sLineBreak +
    'TimeSeparator: %s' + sLineBreak +
    'ListSeparator: %s' + sLineBreak +
    'CurrencyString: %s' + sLineBreak +
    'ShortDateFormat: %s' + sLineBreak +
    'LongDateFormat: %s' + sLineBreak +
    'TimeAMString: %s' + sLineBreak +
    'TimePMString: %s' + sLineBreak +
    'ShortTimeFormat: %s' + sLineBreak +
    'LongTimeFormat: %s' + sLineBreak +
    'ShortMonthNames:  %s' + sLineBreak +
    'LongMonthNames:  %s' + sLineBreak +
    'ShortDayNames:  %s' + sLineBreak +
    'LongDayNames:  %s' + sLineBreak +
    'TwoDigitYearCenturyWindow: %d' + sLineBreak;
var
  Dumme: TDateTime;
  FS: TFormatSettings;
  I: Integer;
  Line, TrimmedMail: String;
  ShortMonthNames, LongMonthNames, ShortDayNames, LongDayNames: String;

begin
  DebugLog.StartLog;
  try
    TrimmedMail := Trim(Value);
    DebugLog.Add('==== Mail data start ====' + sLineBreak +
      TrimmedMail + sLineBreak + '==== Mail data end ====');
    FMail.Text := TrimmedMail;
    Finalize(Result);
    FillChar(Result, SizeOf(TKillMailData), #0);
    DebugLog.Add('Result finalized');
    Result.Status := psInvalid;
    DebugLog.Add('FMail.Count = ' + IntToStr(FMail.Count) + sLineBreak);
    if FMail.Count = 0 then
      Exit;
    GetLocaleFormatSettings(0, FS);

    ShortMonthNames := FS.ShortMonthNames[1];
    for I := 2 to 12 do
      ShortMonthNames := ShortMonthNames + ', ' + FS.ShortMonthNames[I];
    LongMonthNames := FS.LongMonthNames[1];
    for I := 2 to 12 do
      LongMonthNames := LongMonthNames + ', ' + FS.LongMonthNames[I];
    ShortDayNames := FS.ShortDayNames[1];
    for I := 2 to 7 do
      ShortDayNames := ShortDayNames + ', ' + FS.ShortDayNames[I];
    LongDayNames := FS.LongDayNames[1];
    for I := 2 to 7 do
      LongDayNames := LongDayNames + ', ' + FS.LongDayNames[I];
    DebugLog.Add(Format(LFS, [
      FS.CurrencyFormat, FS.NegCurrFormat, FS.ThousandSeparator,
      FS.DecimalSeparator, FS.CurrencyDecimals, FS.DateSeparator,
      FS.TimeSeparator, FS.ListSeparator, FS.CurrencyString,
      FS.ShortDateFormat, FS.LongDateFormat, FS.TimeAMString,
      FS.TimePMString, FS.ShortTimeFormat, FS.LongTimeFormat,
      ShortMonthNames, LongMonthNames, ShortDayNames, LongDayNames,
      FS.TwoDigitYearCenturyWindow]));

    FS.ShortDateFormat := 'yyyy-MM-dd';
    FS.DateSeparator := '.';
    FS.TimeSeparator := ':';
    FS.ShortTimeFormat := 'H.mm';

    DebugLog.Add('Try convert date...');

    if TryStrToDateTime(FMail[0], Dumme, FS) then
    begin
      DebugLog.Add('Success');
      Result.KillDate := FMail[0];
      DebugLog.Add('KillDate: ' + FMail[0]);
      for I := 1 to FMail.Count - 1 do
      begin
        Line := LowerCase(FMail[I]);
        DebugLog.Add('Parse line ' + IntToStr(I) + ', data = "' +
          Line + '"');
        if Copy(Line, 1, Length(INV)) = INV then
        begin
          DebugLog.Add('Parse end');
          Break;
        end;
        if Copy(Line, 1, Length(VIC)) = VIC then
        begin
          Result.Victim := Trim(Copy(Line, Length(VIC) + 1, Length(Line)));
          DebugLog.Add('Victim founded: ' + Result.Victim);
          Result.Status := psKill;
          Continue;
        end;
        TestLine(Line, CORP, FCorporation, ldInCorporate);
        TestLine(Line, ALLI, FAlliance, ldInAlliance);
        TestLine(Line, FACTION, FFaction, ldInFaction);
      end;
    end
    else
      DebugLog.Add('Failed');
  finally
    DebugLog.EndLog;
  end;
end;

{$ELSE}

function TMailParser.ParseMail(const Value: String): TKillMailData;

  procedure TestLine(const Line, Indent, IndentValue: String;
    LooseDetect: TLoosDetect);
  begin
    if Copy(Line, 1, Length(Indent)) = Indent then
      if Trim(Copy(Line, Length(Indent) + 1, Length(Line))) = IndentValue then
        if Result.Status <> psInvalid then
        begin
          Result.Status := psLoose;
          Include(Result.LoosDetect, LooseDetect);
        end;
  end;

const
  INV = 'involved parties';
  VIC = 'victim:';
  CORP = 'corp:';
  ALLI = 'alliance:';
  FACTION = 'faction:';
  MOON = 'moon:';
var
  Dumme: TDateTime;
  FS: TFormatSettings;
  I, A: Integer;
  Line: String;
begin
  // 05.02.2009 Rouse_
  // ver 1.0.4 (Remove empty line in header and footer)
  FMail.Text := Trim(Value);

  Finalize(Result);
  FillChar(Result, SizeOf(TKillMailData), #0);
  Result.Status := psInvalid;
  if FMail.Count = 0 then
    Exit;
  GetLocaleFormatSettings(0, FS);
  FS.ShortDateFormat := 'yyyy-MM-dd';

  // 03.02.2009 Rouse_
  // ver 1.0.2 (Fix regional settings)
  // start
  FS.DateSeparator := '.';
  FS.TimeSeparator := ':';
  FS.ShortTimeFormat := 'H.mm';
  // end

  if TryStrToDateTime(FMail[0], Dumme, FS) then
  begin
    Result.KillDate := FMail[0];
    for I := 1 to FMail.Count - 1 do
    begin
      Line := LowerCase(FMail[I]);
      if Copy(Line, 1, Length(INV)) = INV then
      begin
        // 22.05.2009 Rouse_
        // ver 1.0.9 (Detect NPC killer)
        if Result.Status = psLoose then
        begin
          Result.Status := psNPC;
          for A := I to FMail.Count - 1 do
            if Copy(LowerCase(FMail[A]), 1, Length(ALLI)) = ALLI then
            begin
              Result.Status := psLoose;
              Break;
            end;
        end;
        Break;
      end;
      if Copy(Line, 1, Length(VIC)) = VIC then
      begin
        Result.Victim := Trim(Copy(Line, Length(VIC) + 1, Length(Line)));
        Result.Status := psKill;
        Continue;
      end;
      // 25.02.2009 Rouse_
      // ver 1.0.8 (now we can parse KM from POS Modules)
      if Copy(Line, 1, Length(MOON)) = MOON then
      begin
        Result.Victim := Trim(Copy(Line, Length(MOON) + 1, Length(Line)));
        Result.Status := psKill;
        Continue;
      end;
      TestLine(Line, CORP, FCorporation, ldInCorporate);
      TestLine(Line, ALLI, FAlliance, ldInAlliance);
      TestLine(Line, FACTION, FFaction, ldInFaction);
    end;
  end;
end;

{$ENDIF}

procedure TMailParser.SetCurrentData(const Faction, Alliance,
  Corporation: String);
begin
  FFaction := LowerCase(Faction);
  FAlliance := LowerCase(Alliance);
  FCorporation := LowerCase(Corporation);
end;

initialization

  MailParser := TMailParser.Create;

finalization

  MailParser.Free;

end.
