////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   : EVE Online KillMail Sender
//  * Unit Name : kmsMainDlg
//  * Purpose   :   
//  * Author    :  (Rouse_) 
//  * Copyright :  Fangorn Wizards Lab 1998 - 2009 ()
//  * Version   : 1.09
//  * Home Page : http://rouse.drkb.ru/kms/
//  ****************************************************************************
//

unit kmsMainDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, StdCtrls, ImgList, ComCtrls, ActnList,
  kmsLangSettings, kmsSettings, kmsMailParser, kmsSendThread, kmsHistory,
  ExtCtrls, ToolWin, CommCtrl;

type
  TkmsMainDialog = class(TForm)
    gbKillmail: TGroupBox;
    gbKBList: TGroupBox;
    MainMenu: TMainMenu;
    mnuFile: TMenuItem;
    mnuSetting: TMenuItem;
    N1: TMenuItem;
    mnuExit: TMenuItem;
    mnuHelp: TMenuItem;
    mnuAbout: TMenuItem;
    mnuKillMail: TMenuItem;
    mnuPastefromClipboard: TMenuItem;
    mnuSend: TMenuItem;
    gbLog: TGroupBox;
    N2: TMenuItem;
    mnuLanguage: TMenuItem;
    mnuEnglishDefault: TMenuItem;
    lvKBList: TListView;
    KBLogoImageList: TImageList;
    memLog: TMemo;
    edKillmail: TRichEdit;
    ActionList: TActionList;
    acReadyToSend: TAction;
    pmnuKM: TPopupMenu;
    mnuPaste: TMenuItem;
    pmnuLog: TPopupMenu;
    mnuCopy: TMenuItem;
    gbHistory: TGroupBox;
    lbHistory: TListBox;
    pHistory: TPanel;
    rbShowAll: TRadioButton;
    rbShowSended: TRadioButton;
    pmHistory: TPopupMenu;
    pmHistoryBackup: TMenuItem;
    mnuShowHistory: TMenuItem;
    N3: TMenuItem;
    mnuGroupSend: TMenuItem;
    gbGroupSend: TGroupBox;
    lvGroupSend: TListView;
    OpenDialog: TOpenDialog;
    acStop: TAction;
    Send1: TMenuItem;
    mnuGroupSend1: TMenuItem;
    N4: TMenuItem;
    Send2: TMenuItem;
    mnu_gsenable: TMenuItem;
    ToolBar: TToolBar;
    tbSetting: TToolButton;
    tbShowHistory: TToolButton;
    tbGSEnable: TToolButton;
    ToolButton4: TToolButton;
    tbPaste: TToolButton;
    tbGroupSend: TToolButton;
    tbSend: TToolButton;
    tbStop: TToolButton;
    ToolButton9: TToolButton;
    tbExit: TToolButton;
    TBImageList: TImageList;
    CoolBar: TCoolBar;
    ToolButton1: TToolButton;
    N5: TMenuItem;
    mnuOpenHelp: TMenuItem;
    pmOpenKBInBrowser: TPopupMenu;
    mnuOpenKBInBrowser: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure mnuSettingClick(Sender: TObject);
    procedure mnuExitClick(Sender: TObject);
    procedure mnuPastefromClipboardClick(Sender: TObject);
    procedure mnuCopyClick(Sender: TObject);
    procedure acReadyToSendUpdate(Sender: TObject);
    procedure lvKBListChanging(Sender: TObject; Item: TListItem;
      Change: TItemChange; var AllowChange: Boolean);
    procedure acReadyToSendExecute(Sender: TObject);
    procedure rbShowAllClick(Sender: TObject);
    procedure lbHistoryDblClick(Sender: TObject);
    procedure pmHistoryBackupClick(Sender: TObject);
    procedure mnuShowHistoryClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure mnuGroupSendClick(Sender: TObject);
    procedure acStopUpdate(Sender: TObject);
    procedure acStopExecute(Sender: TObject);
    procedure mnu_gsenableClick(Sender: TObject);
    procedure mnuOpenHelpClick(Sender: TObject);
    procedure mnuOpenKBInBrowserClick(Sender: TObject);
    procedure pmOpenKBInBrowserPopup(Sender: TObject);
  private
    KBIcon: TIcon;
    EmptyIcon: TBitmap;
    CurrentMailData: TKillMailData;
    History: THistoryProvider;
    FThreadCount: Integer;
    SendKBList: TStringList;
    FShowHistory: Boolean;
    FGroupSend: Boolean;
    StopGroupSending: Boolean;
    BlockChangingEvent: Boolean;
    UnknownRequests: TStringList;
    AlphaImgList: HIMAGELIST;
    procedure OnThreadReport(Sender: TObject; const ReportStr: String);
    procedure OnThreadStart(Sender: TObject);
    procedure OnThreadEnd(Sender: TObject);
    procedure OnThreadResult(Sender: TObject;
      const Params: PThreadCreateParams; const Stream: TStream);
    procedure SetThreadCount(const Value: Integer);
  private
    procedure InitFromSetting;
    procedure LoadHistory;
    procedure Log(const Value: String);
    procedure SendAllMails;
    procedure InitImageList;
    procedure ReinitKBList(var Message: TMessage); message WM_USER;
  private
    procedure SetShowHistory(const Value: Boolean);
    procedure SetGroupSend(const Value: Boolean);
    function GetGS: Boolean;
    procedure SetGS(const Value: Boolean);
    function GetShowAllMail: Boolean;
    procedure SetShowAllMail(const Value: Boolean);
  private
    property ThreadCount: Integer read FThreadCount write SetThreadCount;
    property ShowHistory: Boolean read FShowHistory write SetShowHistory;
    property GroupSendReady: Boolean read FGroupSend write SetGroupSend;
    property EnableGroupSend: Boolean read GetGS write SetGS;
    property ShowAllMails: Boolean read GetShowAllMail write SetShowAllMail;
  public
    SettingProvider: TSettingProvider;
    procedure SaveLayout;
  end;

var
  kmsMainDialog: TkmsMainDialog;

implementation

uses
  kmsKBConfig,
  kmsAboutDlg,
  ClipBrd,
  ShellAPI,
  kmsRequestCreator;

const
  StrStatus: array [TMailParserStatus] of String =
    ('Kill', 'Loose', 'NPC Loose', '');

{$R *.dfm}

procedure TkmsMainDialog.acReadyToSendExecute(Sender: TObject);
var
  I: Integer;
  A: TLocalisedString;
  Request: TRequestResult;
  ThreadCreateParam: TThreadCreateParams;
begin
  if GroupSendReady then
  begin
    SendAllMails;
    Exit;
  end;
  SendKBList.Clear;
  UnknownRequests.Clear;
  for I := 0 to lvKBList.Items.Count - 1 do
  begin
    lvKBList.Items[I].SubItems[0] := '';
    if lvKBList.Items[I].Checked then
    begin
      if DWORD(lvKBList.Items[I].Data) = 1 then
        Continue;
      Request := CreateRequest(edKillmail.Text,
        lvKBList.Items[I].Caption, SettingProvider);
      if Request.Done then
      begin
        lvKBList.Items[I].SubItems[0] :=
          SettingProvider.GetLocalisedString(lsWaiting);
        ThreadCreateParam.URL := Request.Host;
        ThreadCreateParam.Headers := Request.Header;
        ThreadCreateParam.Request := Request.RequestData;
        if SettingProvider.ProxyServer <> '' then        
          ThreadCreateParam.Proxy := SettingProvider.ProxyServer +
            ':' + SettingProvider.ProxyPort
        else
          ThreadCreateParam.Proxy := '';
        ThreadCreateParam.ProxyBypass := '';
        ThreadCreateParam.AuthUserName := SettingProvider.ProxyLogin;
        ThreadCreateParam.AuthPassword := SettingProvider.ProxyPassword;
        ThreadCreateParam.KBIndex := I;
        ThreadCreateParam.TemplateIndex := Request.TemplateIndex;
        ThreadCreateParam.OnReport := OnThreadReport;
        ThreadCreateParam.OnResult := OnThreadResult;
        ThreadCreateParam.OnStart := OnThreadStart;
        ThreadCreateParam.OnEnd := OnThreadEnd;
        for A := lsHeaderError to lsSendDone do
          ThreadCreateParam.Localised[A] :=
            SettingProvider.GetLocalisedString(A);
        TSelfResumedSendThread.Create(ThreadCreateParam);
      end
      else
        Log(Request.RequestData);
    end;
  end;
end;

procedure TkmsMainDialog.acReadyToSendUpdate(Sender: TObject);
begin
  if GroupSendReady then
    acReadyToSend.Enabled := StopGroupSending
  else
    acReadyToSend.Enabled :=
      (CurrentMailData.Status <> psInvalid) and
      (ThreadCount = 0);
end;

procedure TkmsMainDialog.acStopExecute(Sender: TObject);
begin
  StopGroupSending := True;
end;

procedure TkmsMainDialog.acStopUpdate(Sender: TObject);
begin
  acStop.Visible := GroupSendReady;
  acStop.Enabled := not StopGroupSending;
end;

procedure TkmsMainDialog.mnuAboutClick(Sender: TObject);
begin
  kmsAboutDialog.ShowModal;
end;

procedure TkmsMainDialog.mnuCopyClick(Sender: TObject);
begin
  Clipboard.AsText := memLog.Text;
end;

procedure TkmsMainDialog.FormCreate(Sender: TObject);

  function GetVersion: String;
  var
    VerInfoSize: DWORD;
    VerInfo: Pointer;
    VerValueSize: DWORD;
    VerValue: PVSFixedFileInfo;
    Dummy: DWORD;
  begin
    VerInfoSize := GetFileVersionInfoSize(PChar(ParamStr(0)), Dummy);
    GetMem(VerInfo, VerInfoSize);
    try
      GetFileVersionInfo(PChar(ParamStr(0)), 0, VerInfoSize, VerInfo);
      VerQueryValue(VerInfo, '\', Pointer(VerValue), VerValueSize);
      with VerValue^ do
      begin
        Result := IntToStr(dwFileVersionMS shr 16);
        Result := Result + '.' + IntToStr(dwFileVersionMS and $FFFF);
        Result := Result + '.' + IntToStr(dwFileVersionLS shr 16);
      end;
    finally
      FreeMem(VerInfo, VerInfoSize);
    end;
  end;

begin
  {$IFDEF DEBUG}
  ReportMemoryLeaksOnShutdown := True;
  {$ENDIF}
  // 04.02.2009 Rouse_
  // ver 1.0.3 (Show version in dialog caption)
  Caption := Caption + GetVersion;
  BlockChangingEvent := False;
  kmsKbConfigDialog := TkmsKbConfigDialog.Create(nil);
  Finalize(CurrentMailData);
  ZeroMemory(@CurrentMailData, SizeOf(TKillMailData));
  CurrentMailData.Status := psInvalid;
  SendKBList := TStringList.Create;
  SettingProvider := TSettingProvider.Create;
  History := THistoryProvider.Create(SettingProvider.SendMailCount);
  ShowHistory := SettingProvider.ShowHistory;
  KBIcon := TIcon.Create;
  EmptyIcon := TBitmap.Create;
  EmptyIcon.Width := 16;
  EmptyIcon.Height := 16;
  InitFromSetting;
  LoadHistory;
  EnableGroupSend := SettingProvider.EnableGroupSend;

  // 03.02.2009 Rouse_
  // ver 1.0.2 (list for additional info)
  UnknownRequests := TStringList.Create;

  // 05.02.2009 Rouse_
  // ver 1.0.4 (Create ImageList)
  InitImageList;
  // ver 1.0.4 (Load history setting)
  ShowAllMails := SettingProvider.ShowAllMails;
end;

procedure TkmsMainDialog.FormDestroy(Sender: TObject);
begin
  SendKBList.Free;
  EmptyIcon.Free;
  KBIcon.Free;
  SettingProvider.Free;
  History.Free;
  kmsKbConfigDialog.Free;
  // 03.02.2009 Rouse_
  // ver 1.0.2 (Release list for additional info)
  UnknownRequests.Free;
  // 05.02.2009 Rouse_
  // ver 1.0.4 (Destroy ImageList)
  ImageList_Destroy(AlphaImgList);  
end;

function TkmsMainDialog.GetGS: Boolean;
begin
  Result := SettingProvider.EnableGroupSend;
end;

function TkmsMainDialog.GetShowAllMail: Boolean;
begin
  Result := SettingProvider.ShowAllMails;
end;

procedure TkmsMainDialog.InitFromSetting;
type
  TLVGROUP = record
    cbSize: UINT;
    mask: UINT;
    pszHeader: LPWSTR;
    cchHeader: Integer;
    pszFooter: LPWSTR;
    cchFooter: Integer;
    iGroupIdL: Integer;
    stateMask: UINT;
    state: UINT;
    uAlign: UINT;
  end;

  tagLVITEMA = packed record
    mask: UINT;
    iItem: Integer;
    iSubItem: Integer;
    state: UINT;
    stateMask: UINT;
    pszText: PAnsiChar;
    cchTextMax: Integer;
    iImage: Integer;
    lParam: lParam;
    iIndent: Integer;
    iGroupId: Integer;
    cColumns: UINT;
    puColumns: PUINT;
  end;
  TLVITEMA = tagLVITEMA;

const
  LVM_INSERTGROUP        = LVM_FIRST + 145;
  LVM_MOVEITEMTOGROUP    = LVM_FIRST + 154;
  LVM_ENABLEGROUPVIEW    = LVM_FIRST + 157;
  LVM_REMOVEALLGROUPS    = LVM_FIRST + 160;

  LVGA_HEADER_LEFT   = $00000001;

  procedure ListView_EnableGroupView(lvHvnd: THandle; fEnable: BOOL);
  begin
    SendMessage(lvHvnd, LVM_ENABLEGROUPVIEW, Integer(fEnable), 0);
  end;

  procedure ListView_InsertGroup(lvHvnd: THandle; szCaption: String;
    byID: Byte; Align: TAlignment = taLeftJustify);
  var
    lvGroup: TLVGROUP;
  begin
    FillChar(lvGroup, SizeOf(TLVGROUP), 0);
    with lvGroup do
    begin
      cbSize := SizeOf(TLVGROUP);
      mask := LVGF_HEADER or LVGF_FOOTER or LVGF_ALIGN or LVGF_GROUPID;
      pszHeader := StringToOleStr(szCaption);
      cchHeader := Length(szCaption);
      iGroupIdL := byID;
      uAlign := LVGA_HEADER_LEFT;
    end;
    SendMessage(lvHvnd, LVM_INSERTGROUP, 0, Longint(@lvGroup));
  end;

  procedure ListView_RemoveAllGroups(lvHvnd: THandle);
  begin
    SendMessage(lvHvnd, LVM_REMOVEALLGROUPS, 0, 0);
  end;

  procedure ListView_MoveItemToGroup(ListView: TListView;
    const ListItem: TListItem; byID: Byte); overload;
  var
    LvItemA: TLVITEMA;
  begin
    FillChar(LvItemA, SizeOf(TLvItemA), 0);
    with LvItemA do
    begin
      mask := LVIF_GROUPID;
      iItem := ListItem.Index;
      iGroupId := byID;
    end;
    SendMessage(ListView.Handle, LVM_SETITEM, 0, Longint(@LvItemA));
  end;

const
  TRUSTED_ID: Byte = 1;
  UNTRUSTED_ID = 2;
var
  I: Integer;
  KBList, UntrustedKBList: TStringList;
  osvi: TOSVersionInfo;
  EnableGroups: Boolean;
  AItem: TListItem;
begin
  lvKBList.Clear;
  KBLogoImageList.Clear;

  // 06.02.2009 Rouse_
  // ver 1.0.5 (Move kb to groups)
  ZeroMemory(@osvi, SizeOf(TOSVersionInfo));
  osvi.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
  if not GetVersionEx(osvi) then Exit;
  EnableGroups := False;
  if osvi.dwMajorVersion > 5 then
    EnableGroups := True
  else
    if (osvi.dwMajorVersion = 5) and (osvi.dwMinorVersion > 0) then
      EnableGroups := True;

  if EnableGroups then
  begin
    ListView_RemoveAllGroups(lvKBList.Handle);
    ListView_EnableGroupView(lvKBList.Handle, True);
    ListView_InsertGroup(lvKBList.Handle,
      SettingProvider.GetLocalisedString(lsUntrustedKB), UNTRUSTED_ID);
    ListView_InsertGroup(lvKBList.Handle,
      SettingProvider.GetLocalisedString(lsTrustedKB), TRUSTED_ID);
  end;

  MailParser.SetCurrentData(
    SettingProvider.FactionDescription,
    SettingProvider.AllianceDescription,
    SettingProvider.CorporationDescription);
  KBList := TStringList.Create;
  try
    UntrustedKBList := TStringList.Create;
    try
      for I := 0 to SettingProvider.KBList.Count - 1 do
      begin
        if SettingProvider.KBList[I].Template = '' then
          Continue;
        if SettingProvider.KBList[I].KBTitle = '' then
          Continue;
        if not SettingProvider.KBList[I].Enabled then
          Continue;

        KBList.Add(SettingProvider.KBList[I].KBTitle);
        if not SettingProvider.KBList[I].SendLooses then
          UntrustedKBList.Add(SettingProvider.KBList[I].KBTitle);

        AItem := lvKBList.Items.Add;
        AItem.Caption := SettingProvider.KBList[I].KBTitle;
        AItem.SubItems.Add('');
        if FileExists(SettingProvider.KBList[I].KBImage) then
        try
          KBIcon.LoadFromFile(SettingProvider.KBList[I].KBImage);
          KBLogoImageList.AddIcon(KBIcon);
        except
          KBLogoImageList.Add(EmptyIcon, nil);
        end;
        AItem.Checked := True;
        AItem.ImageIndex := I;
        if EnableGroups then
        begin
          if SettingProvider.KBList[I].SendLooses then
            ListView_MoveItemToGroup(lvKBList, AItem, TRUSTED_ID)
          else
            ListView_MoveItemToGroup(lvKBList, AItem, UNTRUSTED_ID);
        end;

      end;
      History.SetCurrentKBList(KBList, UntrustedKBList);
    finally
      UntrustedKBList.Free;
    end;
  finally
    KBList.Free;
  end;
end;

procedure TkmsMainDialog.InitImageList;
var
  Icon: TIcon;
  I: Integer;
begin
  AlphaImgList := ImageList_Create(32, 32, ILC_COLOR32, 0, 0);
  Icon := TIcon.Create;
  try
    for I := 1 to 8 do
    begin
      Icon.LoadFromResourceName(HInstance, 'TOOLBAR_ICON' + IntToStr(I));
      ImageList_AddIcon(AlphaImgList, Icon.Handle);
    end;
  finally
    Icon.Free;
  end;
  TBImageList.Handle := AlphaImgList;
end;

procedure TkmsMainDialog.lbHistoryDblClick(Sender: TObject);
var
  I, KBIndex: Integer;
  SendedKB: TStringList;
  Item: TLVItem;
begin
  edKillmail.Lines.LoadFromFile(History[lbHistory.ItemIndex].MailPath);
  CurrentMailData := MailParser.ParseMail(edKillmail.Text);
  SendedKB := TStringList.Create;
  try
    SendedKB.Text := History[lbHistory.ItemIndex].SuccessKBList;
    for I := 0 to lvKBList.Items.Count - 1 do
    begin
      if SendedKB.IndexOf(lvKBList.Items[I].Caption) >= 0 then
      begin
        Item.statemask := LVIS_STATEIMAGEMASK;
        Item.State := 0;
        SendMessage(lvKBList.Handle, LVM_SETITEMSTATE, I, Integer(@Item));
        lvKBList.Items[I].Data := Pointer(1);        
      end
      else
      begin
        lvKBList.Items[I].Data := nil;
        if not History[lbHistory.ItemIndex].Loose then
        begin
          lvKBList.Items[I].Checked := not lvKBList.Items[I].Checked;
          lvKBList.Items[I].Checked := True;
        end
        else
        begin
          KBIndex := SettingProvider.KBList.GetItemIndexByKBTitle(
            lvKBList.Items[I].Caption);
          lvKBList.Items[I].Checked := not lvKBList.Items[I].Checked;
          lvKBList.Items[I].Checked :=
            SettingProvider.KBList[KBIndex].SendLooses;
        end;
      end;
    end;
  finally
    SendedKB.Free;
  end;
end;

procedure TkmsMainDialog.LoadHistory;
var
  I: Integer;
begin
  if not ShowHistory then Exit;  
  lbHistory.Items.BeginUpdate;
  try
    lbHistory.Clear;
    for I := 0 to History.Count - 1 do
      lbHistory.Items.Add(History[I].Victim);
  finally
    lbHistory.Items.EndUpdate;
  end;
end;

procedure TkmsMainDialog.Log(const Value: String);
begin
  memLog.Lines.Add(DateTimeToStr(Now) + '> ' + Value);
end;

procedure TkmsMainDialog.lvKBListChanging(Sender: TObject; Item: TListItem;
  Change: TItemChange; var AllowChange: Boolean);
var
  Info: TLVHitTestInfo;
  KBIndex: Integer;
begin
  if Application.Terminated then
    Exit;

  if GroupSendReady then
    Exit;

  if DWORD(Item.Data) = 1  then
  begin
    AllowChange := False;
    Exit;
  end;
    
  if CurrentMailData.Status <> psLoose then
    Exit;

  if BlockChangingEvent then
    Exit;

  if Change = ctState then
  begin

    KBIndex := SettingProvider.KBList.GetItemIndexByKBTitle(Item.Caption);
    if SettingProvider.KBList[KBIndex].SendLooses then
      Exit;

    GetCursorPos(Info.pt);
    Info.pt := lvKBList.ScreenToClient(Info.pt);
    ListView_HitTest(lvKBList.Handle, Info);
    if (info.flags and LVHT_ONITEMSTATEICON) = LVHT_ONITEMSTATEICON then    
      if not Item.Checked then
        AllowChange := Application.MessageBox(
          PChar(Format(
          SettingProvider.GetLocalisedString(lsLoosoCongirm), [Item.Caption])),
          PChar(SettingProvider.GetLocalisedString(lsConfirm)),
          MB_YESNO or MB_ICONQUESTION or MB_DEFBUTTON2) = IDYES;
  end;
end;

procedure TkmsMainDialog.mnuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TkmsMainDialog.mnuGroupSendClick(Sender: TObject);
var
  I: Integer;
  Path: String;
  S: TStringList;
  Stat: TMailParserStatus;
begin
  if OpenDialog.Execute then
  begin
    lvGroupSend.Clear;
    S := TStringList.Create;
    try
      for I := 0 to OpenDialog.Files.Count - 1 do
      begin
        Path := OpenDialog.Files[I];
        if FileExists(Path) then
        begin
          S.LoadFromFile(Path);
          Stat := MailParser.ParseMail(S.Text).Status;
          case Stat of
            psKill, psLoose:
              with lvGroupSend.Items.Add do
              begin
                Caption := Path;
                SubItems.Add(StrStatus[Stat]);
                SubItems.Add('');
              end;
          end;
        end;
      end;
    finally
      S.Free;
    end;
    GroupSendReady := lvGroupSend.Items.Count > 0;
    StopGroupSending := True;
    acStop.Update;
  end;
end;

procedure TkmsMainDialog.mnuOpenHelpClick(Sender: TObject);
var
  HelpPath: String;
begin
  HelpPath := SettingProvider.HelpFile + '::sources/main.htm';
  HtmlHelp(Handle, PChar(HelpPath), HH_DISPLAY_TOPIC, 0);
end;

procedure TkmsMainDialog.mnuOpenKBInBrowserClick(Sender: TObject);
var
  Request: TRequestResult;
begin
  Request := CreateRequest(edKillmail.Text,
    lvKBList.Items[lvKBList.Selected.Index].Caption, SettingProvider);
  ShellExecute(Handle, nil, PChar(Request.Host), nil, nil, SW_SHOWNORMAL);
end;

procedure TkmsMainDialog.mnuPastefromClipboardClick(Sender: TObject);
var
  I, KBIndex: Integer;
begin
  GroupSendReady := False;
  edKillmail.Text := Clipboard.AsText;
  CurrentMailData := MailParser.ParseMail(edKillmail.Text);
  BlockChangingEvent := True;
  try
    case CurrentMailData.Status of
      psKill:
      begin
        Log(Format(
          SettingProvider.GetLocalisedString(lsPSDone),
            [CurrentMailData.Victim, StrStatus[CurrentMailData.Status]]));
        for I := 0 to lvKBList.Items.Count - 1 do
        begin
          lvKBList.Items[I].Data := nil;
          lvKBList.Items[I].Checked := not lvKBList.Items[I].Checked;
          lvKBList.Items[I].Checked := True;
        end;
      end;
      psLoose:
      begin
        Log(Format(
          SettingProvider.GetLocalisedString(lsPSDone),
            [CurrentMailData.Victim, StrStatus[CurrentMailData.Status]]));
        for I := 0 to lvKBList.Items.Count - 1 do
        begin
          lvKBList.Items[I].Data := nil;
          KBIndex := SettingProvider.KBList.GetItemIndexByKBTitle(
            lvKBList.Items[I].Caption);
          lvKBList.Items[I].Checked := not lvKBList.Items[I].Checked;
          lvKBList.Items[I].Checked :=
            SettingProvider.KBList[KBIndex].SendLooses;
        end;
      end;
      psNPC:
      begin
        Log(Format(
          SettingProvider.GetLocalisedString(lsPSDone),
            [CurrentMailData.Victim, StrStatus[CurrentMailData.Status]]));
        for I := 0 to lvKBList.Items.Count - 1 do
        begin
          lvKBList.Items[I].Data := nil;
          lvKBList.Items[I].Checked := not lvKBList.Items[I].Checked;
          lvKBList.Items[I].Checked := False;
        end;
      end;
      psInvalid:
      begin
        Log(SettingProvider.GetLocalisedString(lsPSError));
        for I := 0 to lvKBList.Items.Count - 1 do
        begin
          lvKBList.Items[I].Data := nil;
          lvKBList.Items[I].Checked := not lvKBList.Items[I].Checked;
          lvKBList.Items[I].Checked := False;
        end;
      end;
    end;
  finally
    BlockChangingEvent := False;
  end; 
end;

procedure TkmsMainDialog.mnuSettingClick(Sender: TObject);
begin
  kmsKbConfigDialog.Settings := SettingProvider;
  if kmsKbConfigDialog.ShowModal = mrOk then
    InitFromSetting;
end;

procedure TkmsMainDialog.mnuShowHistoryClick(Sender: TObject);
begin
  ShowHistory := not ShowHistory;
end;

procedure TkmsMainDialog.mnu_gsenableClick(Sender: TObject);
begin
  EnableGroupSend := not EnableGroupSend;
end;

procedure TkmsMainDialog.pmHistoryBackupClick(Sender: TObject);
var
  I: Integer;
  Path, MailPath: String;
begin
  if Application.MessageBox(
    PChar(SettingProvider.GetLocalisedString(lsMoveToBackup)),
    PChar(SettingProvider.GetLocalisedString(lsConfirm)),
    MB_YESNO or MB_ICONQUESTION or MB_DEFBUTTON2) = IDNO then
    Exit;
  Path := ExtractFilePath(ParamStr(0)) + 'Backup\' + DateToStr(Now) + '\';
  ForceDirectories(Path);
  for I := 0 to lbHistory.Items.Count - 1 do
    if lbHistory.Selected[I] then
    begin
      MailPath := History[I].MailPath;
      MoveFile(PChar(MailPath), PChar(Path + ExtractFileName(MailPath)));
      MailPath := ChangeFileExt(MailPath, '.dat');
      MoveFile(PChar(MailPath), PChar(Path + ExtractFileName(MailPath)));
    end;
  History.ReinitMails;
  LoadHistory;
end;

procedure TkmsMainDialog.pmOpenKBInBrowserPopup(Sender: TObject);
begin
  mnuOpenKBInBrowser.Enabled :=
    Assigned(lvKBList.Selected)  
end;

procedure TkmsMainDialog.OnThreadEnd(Sender: TObject);
begin
  ThreadCount := ThreadCount - 1;
end;

procedure TkmsMainDialog.OnThreadReport(Sender: TObject;
  const ReportStr: String);
begin
  Log(ReportStr);
end;

procedure TkmsMainDialog.OnThreadResult(Sender: TObject;
  const Params: PThreadCreateParams; const Stream: TStream);
var
  WrongPassword, Duplicate, Error, Done, Answer: String;
  AParams: TThreadCreateParams;
  CurrentVictim, UnknownPath, TempPath: String;
  Index, I: Integer;
  M: TMemoryStream;
begin
  AParams := Params^;
  WrongPassword := SettingProvider.KBTemplates[
    AParams.TemplateIndex].GetData('status', 'invalid_password');
  Duplicate := SettingProvider.KBTemplates[
    AParams.TemplateIndex].GetData('status', 'double_post');
  Error := SettingProvider.KBTemplates[
    AParams.TemplateIndex].GetData('status', 'invalid_mail');
  Done := SettingProvider.KBTemplates[
    AParams.TemplateIndex].GetData('status', 'success_text');
  SetLength(Answer, Stream.Size);
  Stream.Position := 0;
  if Stream.Size > 0 then
    Stream.Read(Answer[1], Stream.Size);
  try
    if Pos(WrongPassword, Answer) <> 0 then
    begin
      lvKBList.Items[AParams.KBIndex].SubItems[0] :=
        SettingProvider.GetLocalisedString(lsInvalidPwd1);
      Log(Format(SettingProvider.GetLocalisedString(lsInvalidPwd2), [AParams.Host]));
      Exit;
    end;
    if Pos(Duplicate, Answer) <> 0 then
    begin
      SendKBList.Add(lvKBList.Items[AParams.KBIndex].Caption);
      lvKBList.Items[AParams.KBIndex].SubItems[0] :=
        SettingProvider.GetLocalisedString(lsDuplicate1);
      Log(Format(SettingProvider.GetLocalisedString(lsDuplicate2), [AParams.Host]));
      Exit;
    end;
    if Pos(Done, Answer) <> 0 then
    begin
      SendKBList.Add(lvKBList.Items[AParams.KBIndex].Caption);
      lvKBList.Items[AParams.KBIndex].SubItems[0] :=
        SettingProvider.GetLocalisedString(lsDone1);
      Log(Format(SettingProvider.GetLocalisedString(lsDone2), [AParams.Host]));
      Exit;
    end;
    if Pos(Error, Answer) <> 0 then
    begin
      lvKBList.Items[AParams.KBIndex].SubItems[0] :=
        SettingProvider.GetLocalisedString(lsError1);
      Log(Format(SettingProvider.GetLocalisedString(lsError2), [AParams.Host]));
    end
    else
    begin
      lvKBList.Items[AParams.KBIndex].SubItems[0] :=
        SettingProvider.GetLocalisedString(lsUnknown1);
      Log(Format(SettingProvider.GetLocalisedString(lsUnknown2), [AParams.Host]));
    end;

    // 03.02.2009 Rouse_
    // ver 1.0.2 (Save additional info)
    // ver 1.0.3 (allways)
    //if not GroupSend then
    begin
      M := TMemoryStream.Create;
      try
        M.CopyFrom(Stream, 0);
        UnknownPath := ExtractFilePath(ParamStr(0)) + 'Unknown\';
        ForceDirectories(UnknownPath);
        SetLength(TempPath, MAX_PATH);
        GetTempFileName(PChar(UnknownPath), 'kms', 0, PChar(TempPath));
        TempPath := PChar(TempPath);
        DeleteFile(TempPath);
        TempPath := ChangeFileExt(PChar(TempPath), '.htm');
        M.Position := 0;
        M.SaveToFile(TempPath);
        UnknownRequests.Add(TempPath);
      finally
        M.Free;
      end;
    end;

  finally
    if ThreadCount = 1 then
    begin
      CurrentVictim := Format('%s (%s)',
        [CurrentMailData.Victim, CurrentMailData.KillDate]);
      Index := History.GetVictimIndex(CurrentVictim);
      if Index >= 0 then
        History.ChangeMail(Index, SendKBList)
      else
      begin
        lbHistory.Items.AddObject(CurrentVictim, Pointer(History.Count));
        History.AddMail(edKillmail.Text, CurrentVictim,
          CurrentMailData.Status = psLoose, SendKBList);
        SettingProvider.SendMailCount := History.LastMailName;
      end;

      // 03.02.2009 Rouse_
      // ver 1.0.2 (Show additional info)
      if not GroupSendReady then
        if UnknownRequests.Count > 0 then
          if Application.MessageBox(
            PChar(SettingProvider.GetLocalisedString(lsUnknownAnswer)),
            PChar(SettingProvider.GetLocalisedString(lsError)),
            MB_YESNO or MB_ICONQUESTION or MB_DEFBUTTON1) = IDYES then
            for I := 0 to UnknownRequests.Count - 1 do
              ShellExecute(Handle, nil, PChar(UnknownRequests[I]),
                nil, nil, SW_SHOWNORMAL);
    end;
  end;
end;

procedure TkmsMainDialog.OnThreadStart(Sender: TObject);
begin
  ThreadCount := ThreadCount + 1;
end;

procedure TkmsMainDialog.rbShowAllClick(Sender: TObject);
begin
  History.HideSuccessSendedMails := Boolean(TRadioButton(Sender).Tag);
  LoadHistory;
end;

procedure TkmsMainDialog.ReinitKBList(var Message: TMessage);
begin
  Message.Result := 1;
  if Message.WParam = Integer(Handle) then
    InitFromSetting;
end;

procedure TkmsMainDialog.SaveLayout;
begin
  SettingProvider.SaveSetting;
end;

procedure TkmsMainDialog.SendAllMails;
var
  I, A: Integer;
  Z: TLocalisedString;
  Request: TRequestResult;
  ThreadCreateParam: TThreadCreateParams;
  S: TStringList;
  SendLoose: Boolean;
  SendThread: TSendThread;
begin
  StopGroupSending := False;
  S := TStringList.Create;
  try
    for A := 0 to lvKBList.Items.Count - 1 do
    begin
      if lvKBList.Items[A].Checked then
      begin
        SendLoose := SettingProvider.KBList[A].SendLooses;
        for I := 0 to lvGroupSend.Items.Count - 1 do
        begin
          if StopGroupSending then
            Exit;
          if (lvGroupSend.Items[I].SubItems[0] = StrStatus[psLoose]) then
            if not SendLoose then
              Continue;
          SendKBList.Clear;
          S.LoadFromFile(lvGroupSend.Items[I].Caption);
          edKillmail.Text := S.Text;
          CurrentMailData := MailParser.ParseMail(S.Text);
          Request := CreateRequest(S.Text,
            lvKBList.Items[A].Caption, SettingProvider);
          if Request.Done then
          begin
            lvGroupSend.Items[I].SubItems[1] :=
              SettingProvider.GetLocalisedString(lsWaiting);
            ThreadCreateParam.URL := Request.Host;
            ThreadCreateParam.Headers := Request.Header;
            ThreadCreateParam.Request := Request.RequestData;
            if SettingProvider.ProxyServer <> '' then            
              ThreadCreateParam.Proxy := SettingProvider.ProxyServer +
                ':' + SettingProvider.ProxyPort
            else
              ThreadCreateParam.Proxy := '';
            ThreadCreateParam.ProxyBypass := '';
            ThreadCreateParam.AuthUserName := SettingProvider.ProxyLogin;
            ThreadCreateParam.AuthPassword := SettingProvider.ProxyPassword;
            ThreadCreateParam.KBIndex := A;
            ThreadCreateParam.TemplateIndex := Request.TemplateIndex;
            ThreadCreateParam.OnReport := OnThreadReport;
            ThreadCreateParam.OnResult := OnThreadResult;
            ThreadCreateParam.OnStart := OnThreadStart;
            ThreadCreateParam.OnEnd := OnThreadEnd;
            for Z := lsHeaderError to lsSendDone do
              ThreadCreateParam.Localised[Z] :=
                SettingProvider.GetLocalisedString(Z);
            SendThread := TSendThread.Create(ThreadCreateParam);
            SendThread.Resume;
            while WaitForSingleObject(SendThread.Handle, 10) = WAIT_TIMEOUT do
              Application.HandleMessage;
            lvGroupSend.Items[I].SubItems[1] :=
              lvKBList.Items[A].SubItems[0];
          end
          else
            Log(Request.RequestData);
        end;  
      end;
    end;
  finally
    StopGroupSending := True;
    S.Free;
  end;
end;

procedure TkmsMainDialog.SetGroupSend(const Value: Boolean);
var
  I: Integer;
begin
  FGroupSend := Value;
  gbGroupSend.Visible := Value;
  if Value then
    for I := 0 to lvKBList.Items.Count - 1 do
    begin
      lvKBList.Items[I].Data := nil;
      lvKBList.Items[I].Checked := not lvKBList.Items[I].Checked;
      lvKBList.Items[I].Checked := True;
    end;
end;

procedure TkmsMainDialog.SetGS(const Value: Boolean);
begin
  SettingProvider.EnableGroupSend := Value;
  mnu_gsenable.Checked := Value;
  tbGSEnable.Down := Value;
  mnuGroupSend.Visible := Value;
  mnuGroupSend1.Visible := Value;
  tbGroupSend.Visible := Value;
  if not Value then
    GroupSendReady := False;
  acStop.Update;
end;

procedure TkmsMainDialog.SetShowAllMail(const Value: Boolean);
begin
  SettingProvider.ShowAllMails := Value;
  rbShowAll.Checked := Value;
  rbShowSended.Checked := not Value; 
end;

procedure TkmsMainDialog.SetShowHistory(const Value: Boolean);
begin
  FShowHistory := Value;
  mnuShowHistory.Checked := Value;
  tbShowHistory.Down := Value;
  if Value then
  begin
    gbHistory.Visible := True;
    LoadHistory;
    Width := 998;
  end
  else
  begin
    gbHistory.Visible := False;
    Width := 776;
  end;
  SettingProvider.ShowHistory := Value;
end;

procedure TkmsMainDialog.SetThreadCount(const Value: Integer);
begin
  FThreadCount := Value;
  tbPaste.Enabled := (Value = 0) or (GroupSendReady and StopGroupSending);
  mnuPaste.Enabled := (Value = 0) or (GroupSendReady and StopGroupSending);
  mnuPastefromClipboard.Enabled := (Value = 0) or
    (GroupSendReady and StopGroupSending);
end;

end.
