////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   : EVE Online KillMail Sender
//  * Unit Name : kmsRequestCreator
//  * Purpose   :   POST   ,
//  *           :        .
//  * Author    :  (Rouse_) 
//  * Copyright :  Fangorn Wizards Lab 1998 - 2009 ()
//  * Version   : 1.09
//  * Home Page : http://rouse.drkb.ru/kms/
//  ****************************************************************************
//

unit kmsRequestCreator;

interface

uses
  Classes,
  Windows,
  SysUtils,
  kmsSettings;

type
  TRequestResult = packed record
    Done: Boolean;
    Host: String;
    Header: String;
    RequestData: String;
    TemplateIndex: Integer;
  end;

  function CreateRequest(const Mail, KBTitle: String;
    const Setting: TSettingProvider): TRequestResult;

implementation

uses
  kmsMainDlg,
  kmsLangSettings,
  StrUtils;

function CreateRequest(const Mail, KBTitle: String;
  const Setting: TSettingProvider): TRequestResult;
var
  TemplateIndex: Integer;

  function ExpandString(const Indent, Value: String;
    var Done: Boolean): String;
  var
    I, ByteChar, Len: Integer;
    CharEx, TmpResult: String;
  begin
    Result := '';
    TmpResult := '';
    I := 1;
    while I <= Length(Value) do
    begin
      if Value[I] <> '#' then
        TmpResult := TmpResult + Value[I]
      else
      begin
        CharEx := Copy(Value, I + 1, 2);
        if TryStrToInt('$' + CharEx, ByteChar) then
        begin
          TmpResult := TmpResult + Char(ByteChar);
          Inc(I, 2);
        end
        else
        begin
          CreateRequest.RequestData := Format(
            kmsMainDialog.SettingProvider.GetLocalisedString(lsParserError),
            [Setting.KBTemplates[TemplateIndex].Description,
            Indent, I, CharEx]);
          Exit;
        end;
      end;
      Inc(I);
    end;
    TmpResult := StringReplace(TmpResult, '|', sLineBreak, [rfReplaceAll]);
    SetLength(Result, $7FFF);
    Done := ExpandEnvironmentStrings(PChar(TmpResult),
      @Result[1], $7FFF) <> 0;
    if Done then
    begin
      Result := PChar(Result);
      I := 1;
      I := PosEx('^', Result, I);
      while I > 0 do
      begin
        Len := Length(Result);
        if (I = 1) or (I = Len) then
        begin
          CharEx := '^';
          CreateRequest.RequestData := Format(
            kmsMainDialog.SettingProvider.GetLocalisedString(lsParserError),
            [Setting.KBTemplates[TemplateIndex].Description,
            Indent, I, CharEx]);
          Exit;
        end;
        if Result[I - 1] = Result[I + 1] then
          Delete(Result, I, 2)
        else
          Delete(Result, I, 1);
        I := PosEx('^', Result, I);
      end;
    end;
  end;

  function HTTPEncode(const AStr: String): String;
  const
    NoConversion = ['A'..'Z', 'a'..'z', '*', '@', '.', '_', '-', '0'..'9'];
  var
    Sp, Rp: PChar;
  begin
    SetLength(Result, Length(AStr) * 3);
    Sp := PChar(AStr);
    Rp := PChar(Result);
    while Sp^ <> #0 do
    begin
      if Sp^ in NoConversion then
        Rp^ := Sp^
      else if Sp^ = ' ' then
        Rp^ := '+'
      else
      begin
        FormatBuf(Rp^, 3, '%%%.2x', 6, [Ord(Sp^)]);
        Inc(Rp, 2);
      end;
      Inc(Rp);
      Inc(Sp);
    end;
    SetLength(Result, Rp - PChar(Result));
  end;  

var
  KBIndex: Integer;
  I: Integer;
  ValueName, MailLength, EncodedMail: String;
  EnvironmentList: TStringList;
begin
  Finalize(Result);
  FillChar(Result, SizeOf(TRequestResult), #0);
  KBIndex := Setting.KBList.GetItemIndexByKBTitle(KBTitle);
  TemplateIndex := Setting.KBTemplates.GetItemIndexDescription(
    Setting.KBList[KBIndex].Template);
  Result.TemplateIndex := TemplateIndex;

  EnvironmentList := TStringList.Create;
  try
    EnvironmentList.Text := Setting.KBTemplates[TemplateIndex].Environments +
      Setting.KBList[KBIndex].EnvironmentList;
    for I := 0 to EnvironmentList.Count - 1 do
    begin
      ValueName := EnvironmentList.Names[I];
      SetEnvironmentVariable(PChar(ValueName),
        PChar(EnvironmentList.Values[ValueName]));
    end;

    Result.Host := ExpandString('host',
      Setting.KBTemplates[TemplateIndex].Host, Result.Done);

    if not Result.Done then
    begin
      Result.RequestData := SysErrorMessage(GetLastError);
      Exit;
    end;

    SetEnvironmentVariable('host', PChar(Result.Host));
    try

      EncodedMail := HTTPEncode(Mail);
      Result.RequestData := ExpandString('template',
        Setting.KBTemplates[TemplateIndex].Template, Result.Done);
      if Result.Done then
      begin
        if Pos('%killmail%', Result.RequestData) <> 0 then
        begin
          Result.RequestData :=
            StringReplace(Result.RequestData, '%killmail%',
            EncodedMail, [rfReplaceAll]);
        end
        else
        begin
          Result.Done := False;
          Result.RequestData :=
            Format(kmsMainDialog.SettingProvider.GetLocalisedString(lsParserError),
              [Setting.KBTemplates[TemplateIndex].Description]);
          Exit;
        end;
      end
      else
      begin
        Result.RequestData := SysErrorMessage(GetLastError);
        Exit;
      end;

      MailLength := IntToStr(Length(Result.RequestData));
      SetEnvironmentVariable('mail_len', PChar(MailLength));
      try
        Result.Header := ExpandString('header',
          Setting.KBTemplates[TemplateIndex].Header, Result.Done);
        if not Result.Done then
        begin
          Result.RequestData := SysErrorMessage(GetLastError);
          Exit;
        end;
      finally
        SetEnvironmentVariable('mail_len', nil);
      end;

    finally
      SetEnvironmentVariable('host', nil);
      for I := 0 to EnvironmentList.Count - 1 do
      begin
        ValueName := EnvironmentList.Names[I];
        SetEnvironmentVariable(PChar(ValueName), nil);
      end;
    end;
  finally
    EnvironmentList.Free;
  end; 
end;

end.
