////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   : EVE Online KillMail Sender
//  * Unit Name : kmsSendThread
//  * Purpose   :   ,  
//  *           :   POST   
//  * Author    :  (Rouse_) 
//  * Copyright :  Fangorn Wizards Lab 1998 - 2009 ()
//  * Version   : 1.09
//  * Home Page : http://rouse.drkb.ru/kms/
//  ****************************************************************************
//

unit kmsSendThread;

interface

uses
  Windows,
  SysUtils,
  Classes,
  WinInet,
  kmsLangSettings;

type
  PThreadCreateParams = ^TThreadCreateParams;

  TOnReportEvent = procedure(Sender: TObject;
    const ReportStr: String) of object;
  TOnResultEvent = procedure(Sender: TObject;
    const Params: PThreadCreateParams; const Stream: TStream) of object;

  TThreadCreateParams = record
    // in params
    URL,
    Headers,
    Request,
    Proxy,
    ProxyBypass,
    AuthUserName,
    AuthPassword: String;
    Localised: array [lsHeaderError..lsSendDone] of String;
    KBIndex,
    TemplateIndex: Integer;
    OnReport: TOnReportEvent;
    OnResult: TOnResultEvent;
    OnStart, OnEnd: TNotifyEvent;
    // out param
    Host: String;
  end;

  TSendThread = class(TThread)
  private
    Params: TThreadCreateParams;
    Data: TMemoryStream;
    ReportStr: String;
  protected
    procedure DoStart;
    procedure DoEnd;
    procedure DoReport;
    procedure DoResult;
    procedure Execute; override;
  public
    constructor Create(const Value: TThreadCreateParams); virtual;
    destructor Destroy; override;
  end;

  TSelfResumedSendThread = class(TSendThread)
  public
    constructor Create(const Value: TThreadCreateParams); override;
  end;

implementation

{ TSendThread }

constructor TSendThread.Create(const Value: TThreadCreateParams);
begin
  inherited Create(True);
  Params := Value;
  Data := TMemoryStream.Create;
  FreeOnTerminate := True;
end;

destructor TSendThread.Destroy;
begin
  Data.Free;
  inherited;
end;

procedure TSendThread.DoEnd;
begin
  if Assigned(Params.OnEnd) then
    Params.OnEnd(Self);
end;

procedure TSendThread.DoReport;
begin
  if Assigned(Params.OnReport) then
    Params.OnReport(Self, ReportStr);
end;

procedure TSendThread.DoResult;
begin
  if Assigned(Params.OnResult) then
    Params.OnResult(Self, @Params, Data);
end;

procedure TSendThread.DoStart;
begin
  if Assigned(Params.OnStart) then
    Params.OnStart(Self);
end;

procedure TSendThread.Execute;

 function DelHttp(URL: String): String;
 var
   HttpPos: Integer;
 begin
   HttpPos := Pos('http://', URL);
   if HttpPos > 0 then Delete(Url, HttpPos, 7);
   Result := Copy(Url, 1, Pos('/', Url) - 1);
   if Result = '' then Result := URL;
 end;

var
  FSession, FConnect, FRequest: HINTERNET;
  FHost, FScript, SRequest, ARequest: String;
  Buff, IntermediateBuffer: array of Byte;
  BytesRead, Res, Len,
  OpenTypeFlags, ContentLength: Cardinal;
  Header: TStringList;
  I: Integer;
  ThreadID: DWORD;
begin
  Synchronize(DoStart);
  try
    ThreadID := GetCurrentThreadId;
    ARequest := Params.URL;
    FHost := DelHttp(ARequest);
    FScript := ARequest;
    Delete(FScript, 1, Pos(FHost, FScript) + Length(FHost));
    Params.Host := FHost;

    if Params.Proxy = '' then
     OpenTypeFlags := INTERNET_OPEN_TYPE_PRECONFIG
    else
     OpenTypeFlags := INTERNET_OPEN_TYPE_PROXY;
    FSession := InternetOpen('KMS',
      OpenTypeFlags, PChar(Params.Proxy), PChar(Params.ProxyBypass), 0);

    if not Assigned(FSession) then Exit;
    try
      FConnect := InternetConnect(FSession, PChar(FHost),
        INTERNET_DEFAULT_HTTP_PORT, PChar(Params.AuthUserName),
        PChar(Params.AuthPassword), INTERNET_SERVICE_HTTP, 0, 0);

      if not Assigned(FConnect) then Exit;
      try
        FRequest := HttpOpenRequest(FConnect, 'POST', PChar(FScript), nil,
          nil, nil, INTERNET_FLAG_RELOAD, 0);

        Header := TStringList.Create;
        try
          Header.Text := Params.Headers;
          for I := 1 to Header.Count - 1 do
            if not HttpAddRequestHeaders(FRequest, PChar(Header[I] + sLineBreak),
              Length(Header[I] + sLineBreak),
              HTTP_ADDREQ_FLAG_REPLACE or HTTP_ADDREQ_FLAG_ADD) then
            begin
              ReportStr := Format(Params.Localised[lsHeaderError],
                [Header[I]]);
              Synchronize(DoReport);
              Exit;
            end;
        finally
          Header.Free;
        end;

        Len := 0;
        Res := 0;
        SRequest := ' ';
        HttpQueryInfo(FRequest, HTTP_QUERY_RAW_HEADERS_CRLF or
          HTTP_QUERY_FLAG_REQUEST_HEADERS, @SRequest[1], Len, Res);
        if Len > 0 then
        begin
          SetLength(SRequest, Len);
          HttpQueryInfo(FRequest, HTTP_QUERY_RAW_HEADERS_CRLF or
            HTTP_QUERY_FLAG_REQUEST_HEADERS, @SRequest[1], Len, Res);
        end;

        if not Assigned(FConnect) then Exit;
        try
          ReportStr := Format(Params.Localised[lsTryToSend],
            [ThreadID, FHost]);
          Synchronize(DoReport);

          {$IFDEF DEBUG_EMUL_SEND}
          Sleep(Random(1000));
          {$ELSE}
          if not (HttpSendRequest(FRequest, nil, 0,
            PChar(Params.Request), Length(Params.Request))) then
          begin
            ReportStr := Format(Params.Localised[lsSendError],
              [ThreadID, GetLastError, SysErrorMessage(GetLastError)]);
            Synchronize(DoReport);
            Exit;
          end;
          {$ENDIF}

          ReportStr := Format(Params.Localised[lsSendDone], [ThreadID]);
          Synchronize(DoReport);

          SetLength(IntermediateBuffer, 8192);
          BytesRead := 0;
          ContentLength := BytesRead;
          {$IFDEF DEBUG_EMUL_SEND}
          Data.Size := 0;
          {$ELSE}
          while InternetReadFile(FRequest, @IntermediateBuffer[0],
            1024, BytesRead) do
            if BytesRead > 0 then
            begin
              SetLength(Buff, ContentLength + BytesRead);
              Move(IntermediateBuffer[0], Buff[ContentLength], BytesRead);
              Inc(ContentLength, BytesRead);
            end
            else
              Break;
          {$ENDIF}

          if ContentLength > 0 then          
            Data.Write(Buff[0], ContentLength);
          Synchronize(DoResult);

        finally
          InternetCloseHandle(FRequest);
        end;
      finally
        InternetCloseHandle(FConnect);
      end;
    finally
      InternetCloseHandle(FSession);
    end;
  finally
    Synchronize(DoEnd);
  end;
end;

{ TNoResumedThread }

constructor TSelfResumedSendThread.Create(const Value: TThreadCreateParams);
begin
  inherited Create(Value);
  Resume;
end;

end.
