////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   : EVE Online KillMail Sender
//  * Unit Name : kmsSettings
//  * Purpose   :      
//  * Author    :  (Rouse_) 
//  * Copyright :  Fangorn Wizards Lab 1998 - 2009 ()
//  * Version   : 1.09
//  * Home Page : http://rouse.drkb.ru/kms/
//  ****************************************************************************
//

unit kmsSettings;

interface

uses
  Windows,
  Classes,
  SysUtils,
  IniFiles,
  kmsLangSettings;

const
  SettingVersion = 1;

type
  TSendStatus = (ssSuccess, ssDoublePost, ssInvalidMail, ssWrongPassword);

  TKBTemplate = class(TCollectionItem)
  strict private
    FTPLData: TMemIniFile;
    function GetString(const Index: Integer): String;
  protected
    procedure InitFromTPL(const Value: String);
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    function GetData(const Section, Indent: String): String;
    property Description: String index 0 read GetString;
    property Host: String index 1 read GetString;
    property Header: String index 2 read GetString;
    property Template: String index 3 read GetString;
    property Environments: String index 4 read GetString;
  end;

  TKBTemplates = class(TCollection)
  private
    function GetItem(Index: Integer): TKBTemplate;
    procedure SetItem(Index: Integer; const Value: TKBTemplate);
  public
    function Add: TKBTemplate;
    function GetItemIndexDescription(const Value: String): Integer;
    property Items[Index: Integer]: TKBTemplate read GetItem write SetItem; default;
  end;

  TKBData = class(TCollectionItem)
  strict private
    FKBTitle: String;
    FKBImage: String;
    FTemplate: String;
    FEnvironmentList: String;
    FSendLooses: Boolean;
    FEnabled: Boolean;
    procedure SetString(const Index: Integer; const Value: String);
  public
    constructor Create(Collection: TCollection); override;
    property KBTitle: String index 0 read FKBTitle write SetString;
    property KBImage: String index 1 read FKBImage write SetString;
    property Template: String index 2 read FTemplate write SetString;
    property EnvironmentList: String index 3 read FEnvironmentList write SetString;
    property SendLooses: Boolean read FSendLooses write FSendLooses;
    // 18.02.2009 Rouse_
    // ver 1.0.6 (Disable killboard if needed)
    property Enabled: Boolean read FEnabled write FEnabled;
  end;

  TKBList = class(TCollection)
  private
    function GetItem(Index: Integer): TKBData;
    procedure SetItem(Index: Integer; const Value: TKBData);
  public
    function Add: TKBData;
    function GetItemIndexByKBTitle(const Value: String): Integer;
    property Items[Index: Integer]: TKBData read GetItem write SetItem; default;
  end;

  TSettingProvider = class
  strict private
    FKBList: TKBList;
    FTemplates: TKBTemplates;
    FLangSettings: TLanguageProvider;
    FAllianceDescription: String;
    FCorporationDescription: String;
    FFactionDescription: String;
    FSettingPath: String;
    FProxyPort: String;
    FProxyPassword: String;
    FProxyLogin: String;
    FProxyServer: String;    
    FSendMailCount: Integer;
    FShowHistory: Boolean;
    FEnableGroupSend: Boolean;
    FShowAllMails: Boolean;
    function GetHelpFile: string;
    function GetLanguageID: Integer;
    procedure SetLanguageID(const Value: Integer);
    procedure SetSendMailCount(const Value: Integer);
    procedure SetShowHistory(const Value: Boolean);
    procedure SetEnableGroupSend(const Value: Boolean);
    procedure SetShowAllMails(const Value: Boolean);
  protected
    procedure InitTemplates;
  public
    constructor Create;
    destructor Destroy; override;
    function LoadSetting: Boolean;
    function GetLocalisedString(const Index: TLocalisedString): String;
    procedure SaveSetting;
    property AllianceDescription: String
      read FAllianceDescription write FAllianceDescription;
    property CorporationDescription: String
      read FCorporationDescription write FCorporationDescription;
    property FactionDescription: String read FFactionDescription
      write FFactionDescription;
    property ProxyServer: String read FProxyServer write FProxyServer;
    property ProxyPort: String read FProxyPort write FProxyPort;
    property ProxyLogin: String read FProxyLogin write FProxyLogin;
    property ProxyPassword: String read FProxyPassword write FProxyPassword;
    property KBList: TKBList read FKBList;
    property LanguageID: Integer read GetLanguageID write SetLanguageID;
    property KBTemplates: TKBTemplates read FTemplates;
    property SendMailCount: Integer read FSendMailCount write SetSendMailCount;
    property ShowHistory: Boolean read FShowHistory write SetShowHistory;
    property EnableGroupSend: Boolean read FEnableGroupSend write SetEnableGroupSend;
    property ShowAllMails: Boolean read FShowAllMails write SetShowAllMails;
    property HelpFile: string read GetHelpFile;
  end;

implementation

uses
  ActiveX,
  XMLDoc,
  XMLIntf,
  Forms;    

{ TKBTemplate }

constructor TKBTemplate.Create(Collection: TCollection);
begin
  inherited;
  FTPLData := nil; 
end;

destructor TKBTemplate.Destroy;
begin
  if Assigned(FTPLData) then
    FTPLData.Free;
  inherited;
end;

function TKBTemplate.GetData(const Section, Indent: String): String;
begin
  Result := FTPLData.ReadString(Section, Indent, '');
end;

function TKBTemplate.GetString(const Index: Integer): String;
var
  S: TStringList;
  I: Integer;
begin
  case Index of
    0: Result := GetData('main', 'description');
    1: Result := GetData('main', 'host');
    2: Result := GetData('main', 'header');
    3: Result := GetData('main', 'template');
    4:
    begin
      Result := '';
      S := TStringList.Create;
      try
        FTPLData.ReadSection('constants', S);
        for I := 0 to S.Count - 1 do
          Result := Result + S[I] + '=' +
            GetData('constants', S[I]) + sLineBreak;
      finally
        S.Free;
      end;
    end;
  end;
end;     

procedure TKBTemplate.InitFromTPL(const Value: String);
begin
  if not Assigned(FTPLData) then
    FTPLData := TMemIniFile.Create(Value);
end;

{ TKBTemplates }

function TKBTemplates.Add: TKBTemplate;
begin
  Result := TKBTemplate(inherited Add);
end;

function TKBTemplates.GetItem(Index: Integer): TKBTemplate;
begin
  Result := TKBTemplate(inherited GetItem(Index));
end;

function TKBTemplates.GetItemIndexDescription(const Value: String): Integer;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    if Items[I].Description = Value then
    begin
      Result := I;
      Exit;
    end;
  Result := -1;
end;

procedure TKBTemplates.SetItem(Index: Integer; const Value: TKBTemplate);
begin
  inherited SetItem(Index, Value);
end;

{ TKBData }

constructor TKBData.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  // 24.02.2009 Rouse_
  // ver 1.0.7 (set default state)
  Enabled := True;
end;

procedure TKBData.SetString(const Index: Integer; const Value: String);
begin
  case Index of
    0: FKBTitle := Value;
    1: FKBImage := Value;
    2: FTemplate := Value;
    3: FEnvironmentList := Value;
  end;
end;

{ TKBList }

function TKBList.Add: TKBData;
begin
  Result := TKBData(inherited Add);
end;

function TKBList.GetItem(Index: Integer): TKBData;
begin
  Result := TKBData(inherited GetItem(Index));
end;

function TKBList.GetItemIndexByKBTitle(const Value: String): Integer;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    if Items[I].KBTitle = Value then
    begin
      Result := I;
      Exit;
    end;
  Result := -1;
end;

procedure TKBList.SetItem(Index: Integer; const Value: TKBData);
begin
  inherited SetItem(Index, Value);
end;

{ TSettingProvider }

constructor TSettingProvider.Create;
begin
  FLangSettings := TLanguageProvider.Create;
  FSettingPath := ExtractFilePath(ParamStr(0)) + 'settings.xml';
  FTemplates := TKBTemplates.Create(TKBTemplate);
  InitTemplates;
  FKBList := TKBList.Create(TKBData);
  if not LoadSetting then
    LanguageID := GetKeyboardLayout(GetCurrentThreadId) and $FFFF;
end;

destructor TSettingProvider.Destroy;
begin
  FLangSettings.Free;
  FTemplates.Free;
  FKBList.Free;
  inherited;
end;

function TSettingProvider.GetHelpFile: string;
begin
  Result := FLangSettings.HelpFile;
end;

function TSettingProvider.GetLanguageID: Integer;
begin
  Result := FLangSettings.Layout;
end;

function TSettingProvider.GetLocalisedString(const Index: TLocalisedString): String;
begin
  Result := FLangSettings.LocalisedStrings[Index];
end;

procedure TSettingProvider.InitTemplates;
var
  SR: TSearchRec;
  Path: String;
begin
  KBTemplates.Clear;
  Path := ExtractFilePath(ParamStr(0)) + 'Templates\';
  if FindFirst(Path + '*.tpl', faAnyFile, SR) = 0 then
  try
    repeat
      with KBTemplates.Add do
      begin
        InitFromTPL(Path + SR.Name);
        if Description = '' then
          KBTemplates.Delete(Index);
      end;
    until FindNext(SR) <> 0;
  finally
    FindClose(SR);
  end;
end;

function TSettingProvider.LoadSetting: Boolean;
var
  XMLDocument: IXMLDocument;
  Root, Node: IXMLNode;
  Nodes: IXMLNodeList;
  I: Integer;

  function ReadIndentStr(const Indent: String): String;
  begin
    if Root.HasAttribute(Indent) then
      Result := Root.GetAttributeNS(Indent, '')
    else
      Result := '';
  end;    

begin
  Result := False;
  if FileExists(FSettingPath) then
  begin
    CoInitialize(nil);
    XMLDocument := TXMLDocument.Create(nil);
    XMLDocument.LoadFromFile(FSettingPath);
    if XMLDocument.DocumentElement = nil then Exit;

    Root := XMLDocument.DocumentElement;

    if Root.HasAttribute('locale') then
      LanguageID := Root.GetAttributeNS('locale', '')
    else
      LanguageID := 1033;
    if Root.HasAttribute('mailcnt') then
      FSendMailCount := Root.GetAttributeNS('mailcnt', '')
    else
      FSendMailCount := 0;
    CorporationDescription := ReadIndentStr('corp');
    AllianceDescription := ReadIndentStr('alli');
    FactionDescription := ReadIndentStr('faction');
    ProxyServer := ReadIndentStr('proxy');
    ProxyPort := ReadIndentStr('port');
    ProxyLogin := ReadIndentStr('login');
    ProxyPassword := ReadIndentStr('pwd');
    if Root.HasAttribute('history') then
      FShowHistory := Root.GetAttributeNS('history', '')
    else
      FShowHistory := False;
    if Root.HasAttribute('gs') then
      FEnableGroupSend := Root.GetAttributeNS('gs', '')
    else
      FEnableGroupSend := False;
    // 05.02.2009 Rouse_
    // ver 1.0.4 (New setting)
    if Root.HasAttribute('show_all') then
      FShowAllMails := Root.GetAttributeNS('gs', '')
    else
      FShowAllMails := False;
    FKBList.Clear;
    Nodes := Root.ChildNodes;
    for I := 0 to Nodes.Count - 1 do
    begin
      Node := Nodes.Nodes[I];
      with FKBList.Add do
      begin
        if Node.HasAttribute('kbtitle') then
          KBTitle := Node.GetAttributeNS('kbtitle', '');
        if Node.HasAttribute('kbimage') then
          KBImage := Node.GetAttributeNS('kbimage', '');
        if Node.HasAttribute('tpl') then
          Template := Node.GetAttributeNS('tpl', '');
        if Node.HasAttribute('env') then
          EnvironmentList := Node.GetAttributeNS('env', '');
        SendLooses := False;
        if Node.HasAttribute('send_looses') then
          SendLooses := Node.GetAttributeNS('send_looses', '');
        // 18.02.2009 Rouse_
        // ver 1.0.6 (Disable killboard if needed)
        Enabled := True;
        if Node.HasAttribute('enabled') then
          Enabled := Node.GetAttributeNS('enabled', '');  
      end;
    end;
    Result := True;
  end;
end;

procedure TSettingProvider.SaveSetting;
var
  XMLDocument: IXMLDocument;
  Root, Node: IXMLNode;
  I: Integer;
begin
  XMLDocument := TXMLDocument.Create(Application);
  XMLDocument.Options := XMLDocument.Options + [doNodeAutoIndent];
  if FileExists(FSettingPath) then
    DeleteFile(FSettingPath);
  XMLDocument.Active := True;
  XMLDocument.Version := '1.0';
  XMLDocument.Encoding := 'windows-1251';
  Root := XMLDocument.AddChild('document');
  Root.SetAttributeNS('generator', '', 'KMS');
  Root.SetAttributeNS('version', '', SettingVersion);
  Root.SetAttributeNS('mailcnt', '', SendMailCount);
  Root.SetAttributeNS('locale', '', FLangSettings.Layout);
  Root.SetAttributeNS('corp', '', CorporationDescription);
  Root.SetAttributeNS('alli', '', AllianceDescription);
  Root.SetAttributeNS('faction', '', FactionDescription);
  Root.SetAttributeNS('proxy', '', ProxyServer);
  Root.SetAttributeNS('port', '', ProxyPort);
  Root.SetAttributeNS('login', '', ProxyLogin);
  Root.SetAttributeNS('pwd', '', ProxyPassword);
  Root.SetAttributeNS('history', '', ShowHistory);
  Root.SetAttributeNS('gs', '', EnableGroupSend);
  // 05.02.2009 Rouse_
  // ver 1.0.4 (New setting)
  Root.SetAttributeNS('show_all', '', ShowAllMails);
  for I := 0 to KBList.Count - 1 do
  begin
    Node := Root.AddChild('kb_setting');
    Node.SetAttributeNS('kbtitle', '', KBList[I].KBTitle);
    Node.SetAttributeNS('kbimage', '', KBList[I].KBImage);
    Node.SetAttributeNS('tpl', '', KBList[I].Template);
    Node.SetAttributeNS('env', '', KBList[I].EnvironmentList);
    Node.SetAttributeNS('send_looses', '', KBList[I].SendLooses);
    // 18.02.2009 Rouse_
    // ver 1.0.6 (Disable killboard if needed)
    Node.SetAttributeNS('enabled', '', KBList[I].Enabled);
  end;
  XMLDocument.SaveToFile(FSettingPath);
end;

procedure TSettingProvider.SetEnableGroupSend(const Value: Boolean);
begin
  FEnableGroupSend := Value;
  SaveSetting;
end;

procedure TSettingProvider.SetLanguageID(const Value: Integer);
begin
  FLangSettings.Layout := Value;
end;

procedure TSettingProvider.SetSendMailCount(const Value: Integer);
begin
  FSendMailCount := Value;
  SaveSetting;
end;

procedure TSettingProvider.SetShowAllMails(const Value: Boolean);
begin
  // 05.02.2009 Rouse_
  // ver 1.0.4 (New setting)
  FShowAllMails := Value;
  SaveSetting;
end;

procedure TSettingProvider.SetShowHistory(const Value: Boolean);
begin
  FShowHistory := Value;
  SaveSetting;
end;

end.
